# coding: utf-8

# (C) Copyright IBM Corp. 2020.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""
DNS Zones
"""

from datetime import datetime
from enum import Enum
from typing import Dict, List
import json

from ibm_cloud_sdk_core import BaseService, DetailedResponse
from ibm_cloud_sdk_core.authenticators.authenticator import Authenticator
from ibm_cloud_sdk_core.get_authenticator import get_authenticator_from_environment
from ibm_cloud_sdk_core.utils import datetime_to_string, string_to_datetime

from .common import get_sdk_headers

##############################################################################
# Service
##############################################################################

class DnsZonesV1(BaseService):
    """The DNS Zones V1 service."""

    DEFAULT_SERVICE_URL = 'https://api.dns-svcs.cloud.ibm.com/v1'
    DEFAULT_SERVICE_NAME = 'dns_zones'

    @classmethod
    def new_instance(cls,
                     service_name: str = DEFAULT_SERVICE_NAME,
                    ) -> 'DnsZonesV1':
        """
        Return a new client for the DNS Zones service using the specified
               parameters and external configuration.
        """
        authenticator = get_authenticator_from_environment(service_name)
        service = cls(
            authenticator
            )
        service.configure_service(service_name)
        return service

    def __init__(self,
                 authenticator: Authenticator = None,
                ) -> None:
        """
        Construct a new client for the DNS Zones service.

        :param Authenticator authenticator: The authenticator specifies the authentication mechanism.
               Get up to date information from https://github.com/IBM/python-sdk-core/blob/master/README.md
               about initializing the authenticator of your choice.
        """
        BaseService.__init__(self,
                             service_url=self.DEFAULT_SERVICE_URL,
                             authenticator=authenticator)


    #########################
    # DNS Zones
    #########################


    def list_dnszones(self, instance_id: str, *, x_correlation_id: str = None, offset: int = None, limit: int = None, vpc_id: str = None, **kwargs) -> DetailedResponse:
        """
        List DNS zones.

        List the DNS zones for a given service instance.

        :param str instance_id: The unique identifier of a service instance.
        :param str x_correlation_id: (optional) Uniquely identifying a request.
        :param int offset: (optional) Specify how many DNS zones to skip over, the
               default value is 0.
        :param int limit: (optional) Specify how many DNS zones are returned, the
               default value is 10.
        :param str vpc_id: (optional) Specify the VPC instance id.
        :param dict headers: A `dict` containing the request headers
        :return: A `DetailedResponse` containing the result, headers and HTTP status code.
        :rtype: DetailedResponse with `dict` result representing a `ListDnszones` object
        """

        if instance_id is None:
            raise ValueError('instance_id must be provided')
        headers = {
            'X-Correlation-ID': x_correlation_id
        }
        sdk_headers = get_sdk_headers(service_name=self.DEFAULT_SERVICE_NAME, service_version='V1', operation_id='list_dnszones')
        headers.update(sdk_headers)

        params = {
            'offset': offset,
            'limit': limit,
            'vpc_id': vpc_id
        }

        if 'headers' in kwargs:
            headers.update(kwargs.get('headers'))

        url = '/instances/{0}/dnszones'.format(*self.encode_path_vars(instance_id))
        request = self.prepare_request(method='GET',
                                       url=url,
                                       headers=headers,
                                       params=params)

        response = self.send(request)
        return response


    def create_dnszone(self, instance_id: str, *, name: str = None, description: str = None, label: str = None, x_correlation_id: str = None, **kwargs) -> DetailedResponse:
        """
        Create a DNS zone.

        Create a DNS zone for a given service instance.

        :param str instance_id: The unique identifier of a service instance.
        :param str name: (optional) Name of DNS zone.
        :param str description: (optional) The text describing the purpose of a DNS
               zone.
        :param str label: (optional) The label of a DNS zone.
        :param str x_correlation_id: (optional) Uniquely identifying a request.
        :param dict headers: A `dict` containing the request headers
        :return: A `DetailedResponse` containing the result, headers and HTTP status code.
        :rtype: DetailedResponse with `dict` result representing a `Dnszone` object
        """

        if instance_id is None:
            raise ValueError('instance_id must be provided')
        headers = {
            'X-Correlation-ID': x_correlation_id
        }
        sdk_headers = get_sdk_headers(service_name=self.DEFAULT_SERVICE_NAME, service_version='V1', operation_id='create_dnszone')
        headers.update(sdk_headers)

        data = {
            'name': name,
            'description': description,
            'label': label
        }
        data = {k: v for (k, v) in data.items() if v is not None}
        data = json.dumps(data)
        headers['content-type'] = 'application/json'

        if 'headers' in kwargs:
            headers.update(kwargs.get('headers'))

        url = '/instances/{0}/dnszones'.format(*self.encode_path_vars(instance_id))
        request = self.prepare_request(method='POST',
                                       url=url,
                                       headers=headers,
                                       data=data)

        response = self.send(request)
        return response


    def delete_dnszone(self, instance_id: str, dnszone_id: str, *, x_correlation_id: str = None, **kwargs) -> DetailedResponse:
        """
        Delete a DNS zone.

        Delete a DNS zone.

        :param str instance_id: The unique identifier of a service instance.
        :param str dnszone_id: The unique identifier of a DNS zone.
        :param str x_correlation_id: (optional) Uniquely identifying a request.
        :param dict headers: A `dict` containing the request headers
        :return: A `DetailedResponse` containing the result, headers and HTTP status code.
        :rtype: DetailedResponse
        """

        if instance_id is None:
            raise ValueError('instance_id must be provided')
        if dnszone_id is None:
            raise ValueError('dnszone_id must be provided')
        headers = {
            'X-Correlation-ID': x_correlation_id
        }
        sdk_headers = get_sdk_headers(service_name=self.DEFAULT_SERVICE_NAME, service_version='V1', operation_id='delete_dnszone')
        headers.update(sdk_headers)

        if 'headers' in kwargs:
            headers.update(kwargs.get('headers'))

        url = '/instances/{0}/dnszones/{1}'.format(*self.encode_path_vars(instance_id, dnszone_id))
        request = self.prepare_request(method='DELETE',
                                       url=url,
                                       headers=headers)

        response = self.send(request)
        return response


    def get_dnszone(self, instance_id: str, dnszone_id: str, *, x_correlation_id: str = None, **kwargs) -> DetailedResponse:
        """
        Get a DNS zone.

        Get details of a DNS zone.

        :param str instance_id: The unique identifier of a service instance.
        :param str dnszone_id: The unique identifier of a DNS zone.
        :param str x_correlation_id: (optional) Uniquely identifying a request.
        :param dict headers: A `dict` containing the request headers
        :return: A `DetailedResponse` containing the result, headers and HTTP status code.
        :rtype: DetailedResponse with `dict` result representing a `Dnszone` object
        """

        if instance_id is None:
            raise ValueError('instance_id must be provided')
        if dnszone_id is None:
            raise ValueError('dnszone_id must be provided')
        headers = {
            'X-Correlation-ID': x_correlation_id
        }
        sdk_headers = get_sdk_headers(service_name=self.DEFAULT_SERVICE_NAME, service_version='V1', operation_id='get_dnszone')
        headers.update(sdk_headers)

        if 'headers' in kwargs:
            headers.update(kwargs.get('headers'))

        url = '/instances/{0}/dnszones/{1}'.format(*self.encode_path_vars(instance_id, dnszone_id))
        request = self.prepare_request(method='GET',
                                       url=url,
                                       headers=headers)

        response = self.send(request)
        return response


    def update_dnszone(self, instance_id: str, dnszone_id: str, *, description: str = None, label: str = None, x_correlation_id: str = None, **kwargs) -> DetailedResponse:
        """
        Update the properties of a DNS zone.

        Update the properties of a DNS zone.

        :param str instance_id: The unique identifier of a service instance.
        :param str dnszone_id: The unique identifier of a DNS zone.
        :param str description: (optional) The text describing the purpose of a DNS
               zone.
        :param str label: (optional) The label of a DNS zone.
        :param str x_correlation_id: (optional) Uniquely identifying a request.
        :param dict headers: A `dict` containing the request headers
        :return: A `DetailedResponse` containing the result, headers and HTTP status code.
        :rtype: DetailedResponse with `dict` result representing a `Dnszone` object
        """

        if instance_id is None:
            raise ValueError('instance_id must be provided')
        if dnszone_id is None:
            raise ValueError('dnszone_id must be provided')
        headers = {
            'X-Correlation-ID': x_correlation_id
        }
        sdk_headers = get_sdk_headers(service_name=self.DEFAULT_SERVICE_NAME, service_version='V1', operation_id='update_dnszone')
        headers.update(sdk_headers)

        data = {
            'description': description,
            'label': label
        }
        data = {k: v for (k, v) in data.items() if v is not None}
        data = json.dumps(data)
        headers['content-type'] = 'application/json'

        if 'headers' in kwargs:
            headers.update(kwargs.get('headers'))

        url = '/instances/{0}/dnszones/{1}'.format(*self.encode_path_vars(instance_id, dnszone_id))
        request = self.prepare_request(method='PATCH',
                                       url=url,
                                       headers=headers,
                                       data=data)

        response = self.send(request)
        return response



# coding: utf-8

# (C) Copyright IBM Corp. 2020.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""
Zone Lockdown
"""

from enum import Enum
from typing import Dict, List
import json

from ibm_cloud_sdk_core import BaseService, DetailedResponse
from ibm_cloud_sdk_core.authenticators.authenticator import Authenticator
from ibm_cloud_sdk_core.get_authenticator import get_authenticator_from_environment
from ibm_cloud_sdk_core.utils import convert_model

from .common import get_sdk_headers

##############################################################################
# Service
##############################################################################

class ZoneLockdownV1(BaseService):
    """The Zone Lockdown V1 service."""

    DEFAULT_SERVICE_URL = 'https://api.cis.cloud.ibm.com'
    DEFAULT_SERVICE_NAME = 'zone_lockdown'

    @classmethod
    def new_instance(cls,
                     crn: str,
                     zone_identifier: str,
                     service_name: str = DEFAULT_SERVICE_NAME,
                    ) -> 'ZoneLockdownV1':
        """
        Return a new client for the Zone Lockdown service using the specified
               parameters and external configuration.

        :param str crn: Full crn of the service instance.

        :param str zone_identifier: Zone identifier (zone id).
        """
        if crn is None:
            raise ValueError('crn must be provided')
        if zone_identifier is None:
            raise ValueError('zone_identifier must be provided')

        authenticator = get_authenticator_from_environment(service_name)
        service = cls(
            crn,
            zone_identifier,
            authenticator
            )
        service.configure_service(service_name)
        return service

    def __init__(self,
                 crn: str,
                 zone_identifier: str,
                 authenticator: Authenticator = None,
                ) -> None:
        """
        Construct a new client for the Zone Lockdown service.

        :param str crn: Full crn of the service instance.

        :param str zone_identifier: Zone identifier (zone id).

        :param Authenticator authenticator: The authenticator specifies the authentication mechanism.
               Get up to date information from https://github.com/IBM/python-sdk-core/blob/master/README.md
               about initializing the authenticator of your choice.
        """
        if crn is None:
            raise ValueError('crn must be provided')
        if zone_identifier is None:
            raise ValueError('zone_identifier must be provided')

        BaseService.__init__(self,
                             service_url=self.DEFAULT_SERVICE_URL,
                             authenticator=authenticator)
        self.crn = crn
        self.zone_identifier = zone_identifier


    #########################
    # Zone Lockdown Rules
    #########################


    def list_all_zone_lockown_rules(self,
        *,
        page: int = None,
        per_page: int = None,
        **kwargs
    ) -> DetailedResponse:
        """
        List all lockdown rules.

        List all lockdown rules for a zone.

        :param int page: (optional) Page number of paginated results.
        :param int per_page: (optional) Maximum number of lockdown rules per page.
        :param dict headers: A `dict` containing the request headers
        :return: A `DetailedResponse` containing the result, headers and HTTP status code.
        :rtype: DetailedResponse with `dict` result representing a `ListLockdownResp` object
        """

        headers = {}
        sdk_headers = get_sdk_headers(service_name=self.DEFAULT_SERVICE_NAME,
                                      service_version='V1',
                                      operation_id='list_all_zone_lockown_rules')
        headers.update(sdk_headers)

        params = {
            'page': page,
            'per_page': per_page
        }

        if 'headers' in kwargs:
            headers.update(kwargs.get('headers'))

        url = '/v1/{0}/zones/{1}/firewall/lockdowns'.format(
            *self.encode_path_vars(self.crn, self.zone_identifier))
        request = self.prepare_request(method='GET',
                                       url=url,
                                       headers=headers,
                                       params=params)

        response = self.send(request)
        return response


    def create_zone_lockdown_rule(self,
        *,
        urls: List[str] = None,
        configurations: List['LockdownInputConfigurationsItem'] = None,
        id: str = None,
        paused: bool = None,
        description: str = None,
        **kwargs
    ) -> DetailedResponse:
        """
        Create lockdown rule.

        Create a new lockdown rule for a given zone under a service instance.

        :param List[str] urls: (optional) URLs to be included in this rule
               definition. Wildcards are permitted. The URL pattern entered here will be
               escaped before use. This limits the URL to just simple wildcard patterns.
        :param List[LockdownInputConfigurationsItem] configurations: (optional)
               List of IP addresses or CIDR ranges to use for this rule. This can include
               any number of ip or ip_range configurations that can access the provided
               URLs.
        :param str id: (optional) Lockdown rule identifier.
        :param bool paused: (optional) Whether this zone lockdown is currently
               paused.
        :param str description: (optional) A note that you can use to describe the
               reason for a Lockdown rule.
        :param dict headers: A `dict` containing the request headers
        :return: A `DetailedResponse` containing the result, headers and HTTP status code.
        :rtype: DetailedResponse with `dict` result representing a `LockdownResp` object
        """

        if configurations is not None:
            configurations = [convert_model(x) for x in configurations]
        headers = {}
        sdk_headers = get_sdk_headers(service_name=self.DEFAULT_SERVICE_NAME,
                                      service_version='V1',
                                      operation_id='create_zone_lockdown_rule')
        headers.update(sdk_headers)

        data = {
            'urls': urls,
            'configurations': configurations,
            'id': id,
            'paused': paused,
            'description': description
        }
        data = {k: v for (k, v) in data.items() if v is not None}
        data = json.dumps(data)
        headers['content-type'] = 'application/json'

        if 'headers' in kwargs:
            headers.update(kwargs.get('headers'))

        url = '/v1/{0}/zones/{1}/firewall/lockdowns'.format(
            *self.encode_path_vars(self.crn, self.zone_identifier))
        request = self.prepare_request(method='POST',
                                       url=url,
                                       headers=headers,
                                       data=data)

        response = self.send(request)
        return response


    def delete_zone_lockdown_rule(self,
        lockdown_rule_identifier: str,
        **kwargs
    ) -> DetailedResponse:
        """
        Delete lockdown rule.

        Delete a lockdown rule for a particular zone, given its id.

        :param str lockdown_rule_identifier: Identifier of the lockdown rule to be
               deleted.
        :param dict headers: A `dict` containing the request headers
        :return: A `DetailedResponse` containing the result, headers and HTTP status code.
        :rtype: DetailedResponse with `dict` result representing a `DeleteLockdownResp` object
        """

        if lockdown_rule_identifier is None:
            raise ValueError('lockdown_rule_identifier must be provided')
        headers = {}
        sdk_headers = get_sdk_headers(service_name=self.DEFAULT_SERVICE_NAME,
                                      service_version='V1',
                                      operation_id='delete_zone_lockdown_rule')
        headers.update(sdk_headers)

        if 'headers' in kwargs:
            headers.update(kwargs.get('headers'))

        url = '/v1/{0}/zones/{1}/firewall/lockdowns/{2}'.format(
            *self.encode_path_vars(self.crn, self.zone_identifier, lockdown_rule_identifier))
        request = self.prepare_request(method='DELETE',
                                       url=url,
                                       headers=headers)

        response = self.send(request)
        return response


    def get_lockdown(self,
        lockdown_rule_identifier: str,
        **kwargs
    ) -> DetailedResponse:
        """
        Get lockdown rule.

        For a given service instance, zone id and lockdown rule id, get the lockdown rule
        details.

        :param str lockdown_rule_identifier: Identifier of lockdown rule for the
               given zone.
        :param dict headers: A `dict` containing the request headers
        :return: A `DetailedResponse` containing the result, headers and HTTP status code.
        :rtype: DetailedResponse with `dict` result representing a `LockdownResp` object
        """

        if lockdown_rule_identifier is None:
            raise ValueError('lockdown_rule_identifier must be provided')
        headers = {}
        sdk_headers = get_sdk_headers(service_name=self.DEFAULT_SERVICE_NAME,
                                      service_version='V1',
                                      operation_id='get_lockdown')
        headers.update(sdk_headers)

        if 'headers' in kwargs:
            headers.update(kwargs.get('headers'))

        url = '/v1/{0}/zones/{1}/firewall/lockdowns/{2}'.format(
            *self.encode_path_vars(self.crn, self.zone_identifier, lockdown_rule_identifier))
        request = self.prepare_request(method='GET',
                                       url=url,
                                       headers=headers)

        response = self.send(request)
        return response


    def update_lockdown_rule(self,
        lockdown_rule_identifier: str,
        *,
        urls: List[str] = None,
        configurations: List['LockdownInputConfigurationsItem'] = None,
        id: str = None,
        paused: bool = None,
        description: str = None,
        **kwargs
    ) -> DetailedResponse:
        """
        Update lockdown rule.

        Update an existing lockdown rule for a given zone under a given service instance.

        :param str lockdown_rule_identifier: Identifier of lockdown rule.
        :param List[str] urls: (optional) URLs to be included in this rule
               definition. Wildcards are permitted. The URL pattern entered here will be
               escaped before use. This limits the URL to just simple wildcard patterns.
        :param List[LockdownInputConfigurationsItem] configurations: (optional)
               List of IP addresses or CIDR ranges to use for this rule. This can include
               any number of ip or ip_range configurations that can access the provided
               URLs.
        :param str id: (optional) Lockdown rule identifier.
        :param bool paused: (optional) Whether this zone lockdown is currently
               paused.
        :param str description: (optional) A note that you can use to describe the
               reason for a Lockdown rule.
        :param dict headers: A `dict` containing the request headers
        :return: A `DetailedResponse` containing the result, headers and HTTP status code.
        :rtype: DetailedResponse with `dict` result representing a `LockdownResp` object
        """

        if lockdown_rule_identifier is None:
            raise ValueError('lockdown_rule_identifier must be provided')
        if configurations is not None:
            configurations = [convert_model(x) for x in configurations]
        headers = {}
        sdk_headers = get_sdk_headers(service_name=self.DEFAULT_SERVICE_NAME,
                                      service_version='V1',
                                      operation_id='update_lockdown_rule')
        headers.update(sdk_headers)

        data = {
            'urls': urls,
            'configurations': configurations,
            'id': id,
            'paused': paused,
            'description': description
        }
        data = {k: v for (k, v) in data.items() if v is not None}
        data = json.dumps(data)
        headers['content-type'] = 'application/json'

        if 'headers' in kwargs:
            headers.update(kwargs.get('headers'))

        url = '/v1/{0}/zones/{1}/firewall/lockdowns/{2}'.format(
            *self.encode_path_vars(self.crn, self.zone_identifier, lockdown_rule_identifier))
        request = self.prepare_request(method='PUT',
                                       url=url,
                                       headers=headers,
                                       data=data)

        response = self.send(request)
        return response


##############################################################################
# Models
##############################################################################


class DeleteLockdownRespResult():
    """
    Container for response information.

    :attr str id: ID.
    """

    def __init__(self,
                 id: str) -> None:
        """
        Initialize a DeleteLockdownRespResult object.

        :param str id: ID.
        """
        self.id = id

    @classmethod
    def from_dict(cls, _dict: Dict) -> 'DeleteLockdownRespResult':
        """Initialize a DeleteLockdownRespResult object from a json dictionary."""
        args = {}
        if 'id' in _dict:
            args['id'] = _dict.get('id')
        else:
            raise ValueError('Required property \'id\' not present in DeleteLockdownRespResult JSON')
        return cls(**args)

    @classmethod
    def _from_dict(cls, _dict):
        """Initialize a DeleteLockdownRespResult object from a json dictionary."""
        return cls.from_dict(_dict)

    def to_dict(self) -> Dict:
        """Return a json dictionary representing this model."""
        _dict = {}
        if hasattr(self, 'id') and self.id is not None:
            _dict['id'] = self.id
        return _dict

    def _to_dict(self):
        """Return a json dictionary representing this model."""
        return self.to_dict()

    def __str__(self) -> str:
        """Return a `str` version of this DeleteLockdownRespResult object."""
        return json.dumps(self.to_dict(), indent=2)

    def __eq__(self, other: 'DeleteLockdownRespResult') -> bool:
        """Return `true` when self and other are equal, false otherwise."""
        if not isinstance(other, self.__class__):
            return False
        return self.__dict__ == other.__dict__

    def __ne__(self, other: 'DeleteLockdownRespResult') -> bool:
        """Return `true` when self and other are not equal, false otherwise."""
        return not self == other

class ListLockdownRespResultInfo():
    """
    Statistics of results.

    :attr int page: Page number.
    :attr int per_page: Number of results per page.
    :attr int count: Number of results.
    :attr int total_count: Total number of results.
    """

    def __init__(self,
                 page: int,
                 per_page: int,
                 count: int,
                 total_count: int) -> None:
        """
        Initialize a ListLockdownRespResultInfo object.

        :param int page: Page number.
        :param int per_page: Number of results per page.
        :param int count: Number of results.
        :param int total_count: Total number of results.
        """
        self.page = page
        self.per_page = per_page
        self.count = count
        self.total_count = total_count

    @classmethod
    def from_dict(cls, _dict: Dict) -> 'ListLockdownRespResultInfo':
        """Initialize a ListLockdownRespResultInfo object from a json dictionary."""
        args = {}
        if 'page' in _dict:
            args['page'] = _dict.get('page')
        else:
            raise ValueError('Required property \'page\' not present in ListLockdownRespResultInfo JSON')
        if 'per_page' in _dict:
            args['per_page'] = _dict.get('per_page')
        else:
            raise ValueError('Required property \'per_page\' not present in ListLockdownRespResultInfo JSON')
        if 'count' in _dict:
            args['count'] = _dict.get('count')
        else:
            raise ValueError('Required property \'count\' not present in ListLockdownRespResultInfo JSON')
        if 'total_count' in _dict:
            args['total_count'] = _dict.get('total_count')
        else:
            raise ValueError('Required property \'total_count\' not present in ListLockdownRespResultInfo JSON')
        return cls(**args)

    @classmethod
    def _from_dict(cls, _dict):
        """Initialize a ListLockdownRespResultInfo object from a json dictionary."""
        return cls.from_dict(_dict)

    def to_dict(self) -> Dict:
        """Return a json dictionary representing this model."""
        _dict = {}
        if hasattr(self, 'page') and self.page is not None:
            _dict['page'] = self.page
        if hasattr(self, 'per_page') and self.per_page is not None:
            _dict['per_page'] = self.per_page
        if hasattr(self, 'count') and self.count is not None:
            _dict['count'] = self.count
        if hasattr(self, 'total_count') and self.total_count is not None:
            _dict['total_count'] = self.total_count
        return _dict

    def _to_dict(self):
        """Return a json dictionary representing this model."""
        return self.to_dict()

    def __str__(self) -> str:
        """Return a `str` version of this ListLockdownRespResultInfo object."""
        return json.dumps(self.to_dict(), indent=2)

    def __eq__(self, other: 'ListLockdownRespResultInfo') -> bool:
        """Return `true` when self and other are equal, false otherwise."""
        if not isinstance(other, self.__class__):
            return False
        return self.__dict__ == other.__dict__

    def __ne__(self, other: 'ListLockdownRespResultInfo') -> bool:
        """Return `true` when self and other are not equal, false otherwise."""
        return not self == other

class LockdownInputConfigurationsItem():
    """
    LockdownInputConfigurationsItem.

    :attr str target: properties.
    :attr str value: IP addresses or CIDR, if target is "ip", then value should be
          an IP addresses, otherwise CIDR.
    """

    def __init__(self,
                 target: str,
                 value: str) -> None:
        """
        Initialize a LockdownInputConfigurationsItem object.

        :param str target: properties.
        :param str value: IP addresses or CIDR, if target is "ip", then value
               should be an IP addresses, otherwise CIDR.
        """
        self.target = target
        self.value = value

    @classmethod
    def from_dict(cls, _dict: Dict) -> 'LockdownInputConfigurationsItem':
        """Initialize a LockdownInputConfigurationsItem object from a json dictionary."""
        args = {}
        if 'target' in _dict:
            args['target'] = _dict.get('target')
        else:
            raise ValueError('Required property \'target\' not present in LockdownInputConfigurationsItem JSON')
        if 'value' in _dict:
            args['value'] = _dict.get('value')
        else:
            raise ValueError('Required property \'value\' not present in LockdownInputConfigurationsItem JSON')
        return cls(**args)

    @classmethod
    def _from_dict(cls, _dict):
        """Initialize a LockdownInputConfigurationsItem object from a json dictionary."""
        return cls.from_dict(_dict)

    def to_dict(self) -> Dict:
        """Return a json dictionary representing this model."""
        _dict = {}
        if hasattr(self, 'target') and self.target is not None:
            _dict['target'] = self.target
        if hasattr(self, 'value') and self.value is not None:
            _dict['value'] = self.value
        return _dict

    def _to_dict(self):
        """Return a json dictionary representing this model."""
        return self.to_dict()

    def __str__(self) -> str:
        """Return a `str` version of this LockdownInputConfigurationsItem object."""
        return json.dumps(self.to_dict(), indent=2)

    def __eq__(self, other: 'LockdownInputConfigurationsItem') -> bool:
        """Return `true` when self and other are equal, false otherwise."""
        if not isinstance(other, self.__class__):
            return False
        return self.__dict__ == other.__dict__

    def __ne__(self, other: 'LockdownInputConfigurationsItem') -> bool:
        """Return `true` when self and other are not equal, false otherwise."""
        return not self == other

    class TargetEnum(str, Enum):
        """
        properties.
        """
        IP = 'ip'
        IP_RANGE = 'ip_range'


class LockdownObjectConfigurationsItem():
    """
    LockdownObjectConfigurationsItem.

    :attr str target: target.
    :attr str value: IP addresses or CIDR, if target is "ip", then value should be
          an IP addresses, otherwise CIDR.
    """

    def __init__(self,
                 target: str,
                 value: str) -> None:
        """
        Initialize a LockdownObjectConfigurationsItem object.

        :param str target: target.
        :param str value: IP addresses or CIDR, if target is "ip", then value
               should be an IP addresses, otherwise CIDR.
        """
        self.target = target
        self.value = value

    @classmethod
    def from_dict(cls, _dict: Dict) -> 'LockdownObjectConfigurationsItem':
        """Initialize a LockdownObjectConfigurationsItem object from a json dictionary."""
        args = {}
        if 'target' in _dict:
            args['target'] = _dict.get('target')
        else:
            raise ValueError('Required property \'target\' not present in LockdownObjectConfigurationsItem JSON')
        if 'value' in _dict:
            args['value'] = _dict.get('value')
        else:
            raise ValueError('Required property \'value\' not present in LockdownObjectConfigurationsItem JSON')
        return cls(**args)

    @classmethod
    def _from_dict(cls, _dict):
        """Initialize a LockdownObjectConfigurationsItem object from a json dictionary."""
        return cls.from_dict(_dict)

    def to_dict(self) -> Dict:
        """Return a json dictionary representing this model."""
        _dict = {}
        if hasattr(self, 'target') and self.target is not None:
            _dict['target'] = self.target
        if hasattr(self, 'value') and self.value is not None:
            _dict['value'] = self.value
        return _dict

    def _to_dict(self):
        """Return a json dictionary representing this model."""
        return self.to_dict()

    def __str__(self) -> str:
        """Return a `str` version of this LockdownObjectConfigurationsItem object."""
        return json.dumps(self.to_dict(), indent=2)

    def __eq__(self, other: 'LockdownObjectConfigurationsItem') -> bool:
        """Return `true` when self and other are equal, false otherwise."""
        if not isinstance(other, self.__class__):
            return False
        return self.__dict__ == other.__dict__

    def __ne__(self, other: 'LockdownObjectConfigurationsItem') -> bool:
        """Return `true` when self and other are not equal, false otherwise."""
        return not self == other

    class TargetEnum(str, Enum):
        """
        target.
        """
        IP = 'ip'
        IP_RANGE = 'ip_range'


class DeleteLockdownResp():
    """
    delete lockdown response.

    :attr bool success: Operation success flag.
    :attr List[List[str]] errors: Array of errors encountered.
    :attr List[List[str]] messages: Array of messages returned.
    :attr DeleteLockdownRespResult result: Container for response information.
    """

    def __init__(self,
                 success: bool,
                 errors: List[List[str]],
                 messages: List[List[str]],
                 result: 'DeleteLockdownRespResult') -> None:
        """
        Initialize a DeleteLockdownResp object.

        :param bool success: Operation success flag.
        :param List[List[str]] errors: Array of errors encountered.
        :param List[List[str]] messages: Array of messages returned.
        :param DeleteLockdownRespResult result: Container for response information.
        """
        self.success = success
        self.errors = errors
        self.messages = messages
        self.result = result

    @classmethod
    def from_dict(cls, _dict: Dict) -> 'DeleteLockdownResp':
        """Initialize a DeleteLockdownResp object from a json dictionary."""
        args = {}
        if 'success' in _dict:
            args['success'] = _dict.get('success')
        else:
            raise ValueError('Required property \'success\' not present in DeleteLockdownResp JSON')
        if 'errors' in _dict:
            args['errors'] = _dict.get('errors')
        else:
            raise ValueError('Required property \'errors\' not present in DeleteLockdownResp JSON')
        if 'messages' in _dict:
            args['messages'] = _dict.get('messages')
        else:
            raise ValueError('Required property \'messages\' not present in DeleteLockdownResp JSON')
        if 'result' in _dict:
            args['result'] = DeleteLockdownRespResult.from_dict(_dict.get('result'))
        else:
            raise ValueError('Required property \'result\' not present in DeleteLockdownResp JSON')
        return cls(**args)

    @classmethod
    def _from_dict(cls, _dict):
        """Initialize a DeleteLockdownResp object from a json dictionary."""
        return cls.from_dict(_dict)

    def to_dict(self) -> Dict:
        """Return a json dictionary representing this model."""
        _dict = {}
        if hasattr(self, 'success') and self.success is not None:
            _dict['success'] = self.success
        if hasattr(self, 'errors') and self.errors is not None:
            _dict['errors'] = self.errors
        if hasattr(self, 'messages') and self.messages is not None:
            _dict['messages'] = self.messages
        if hasattr(self, 'result') and self.result is not None:
            _dict['result'] = self.result.to_dict()
        return _dict

    def _to_dict(self):
        """Return a json dictionary representing this model."""
        return self.to_dict()

    def __str__(self) -> str:
        """Return a `str` version of this DeleteLockdownResp object."""
        return json.dumps(self.to_dict(), indent=2)

    def __eq__(self, other: 'DeleteLockdownResp') -> bool:
        """Return `true` when self and other are equal, false otherwise."""
        if not isinstance(other, self.__class__):
            return False
        return self.__dict__ == other.__dict__

    def __ne__(self, other: 'DeleteLockdownResp') -> bool:
        """Return `true` when self and other are not equal, false otherwise."""
        return not self == other

class ListLockdownResp():
    """
    list lockdown response.

    :attr bool success: Was operation successful.
    :attr List[List[str]] errors: Array of errors encountered.
    :attr List[List[str]] messages: Array of messages returned.
    :attr List[LockdownObject] result: Container for response information.
    :attr ListLockdownRespResultInfo result_info: Statistics of results.
    """

    def __init__(self,
                 success: bool,
                 errors: List[List[str]],
                 messages: List[List[str]],
                 result: List['LockdownObject'],
                 result_info: 'ListLockdownRespResultInfo') -> None:
        """
        Initialize a ListLockdownResp object.

        :param bool success: Was operation successful.
        :param List[List[str]] errors: Array of errors encountered.
        :param List[List[str]] messages: Array of messages returned.
        :param List[LockdownObject] result: Container for response information.
        :param ListLockdownRespResultInfo result_info: Statistics of results.
        """
        self.success = success
        self.errors = errors
        self.messages = messages
        self.result = result
        self.result_info = result_info

    @classmethod
    def from_dict(cls, _dict: Dict) -> 'ListLockdownResp':
        """Initialize a ListLockdownResp object from a json dictionary."""
        args = {}
        if 'success' in _dict:
            args['success'] = _dict.get('success')
        else:
            raise ValueError('Required property \'success\' not present in ListLockdownResp JSON')
        if 'errors' in _dict:
            args['errors'] = _dict.get('errors')
        else:
            raise ValueError('Required property \'errors\' not present in ListLockdownResp JSON')
        if 'messages' in _dict:
            args['messages'] = _dict.get('messages')
        else:
            raise ValueError('Required property \'messages\' not present in ListLockdownResp JSON')
        if 'result' in _dict:
            args['result'] = [LockdownObject.from_dict(x) for x in _dict.get('result')]
        else:
            raise ValueError('Required property \'result\' not present in ListLockdownResp JSON')
        if 'result_info' in _dict:
            args['result_info'] = ListLockdownRespResultInfo.from_dict(_dict.get('result_info'))
        else:
            raise ValueError('Required property \'result_info\' not present in ListLockdownResp JSON')
        return cls(**args)

    @classmethod
    def _from_dict(cls, _dict):
        """Initialize a ListLockdownResp object from a json dictionary."""
        return cls.from_dict(_dict)

    def to_dict(self) -> Dict:
        """Return a json dictionary representing this model."""
        _dict = {}
        if hasattr(self, 'success') and self.success is not None:
            _dict['success'] = self.success
        if hasattr(self, 'errors') and self.errors is not None:
            _dict['errors'] = self.errors
        if hasattr(self, 'messages') and self.messages is not None:
            _dict['messages'] = self.messages
        if hasattr(self, 'result') and self.result is not None:
            _dict['result'] = [x.to_dict() for x in self.result]
        if hasattr(self, 'result_info') and self.result_info is not None:
            _dict['result_info'] = self.result_info.to_dict()
        return _dict

    def _to_dict(self):
        """Return a json dictionary representing this model."""
        return self.to_dict()

    def __str__(self) -> str:
        """Return a `str` version of this ListLockdownResp object."""
        return json.dumps(self.to_dict(), indent=2)

    def __eq__(self, other: 'ListLockdownResp') -> bool:
        """Return `true` when self and other are equal, false otherwise."""
        if not isinstance(other, self.__class__):
            return False
        return self.__dict__ == other.__dict__

    def __ne__(self, other: 'ListLockdownResp') -> bool:
        """Return `true` when self and other are not equal, false otherwise."""
        return not self == other

class LockdownObject():
    """
    lockdown object.

    :attr str id: Lockdown rule identifier.
    :attr bool paused: Whether this zone lockdown is currently paused.
    :attr str description: A note that you can use to describe the reason for a
          Lockdown rule.
    :attr List[str] urls: URLs to be included in this rule definition. Wildcards are
          permitted. The URL pattern entered here will be escaped before use. This limits
          the URL to just simple wildcard patterns.
    :attr List[LockdownObjectConfigurationsItem] configurations: List of IP
          addresses or CIDR ranges to use for this rule. This can include any number of ip
          or ip_range configurations that can access the provided URLs.
    """

    def __init__(self,
                 id: str,
                 paused: bool,
                 description: str,
                 urls: List[str],
                 configurations: List['LockdownObjectConfigurationsItem']) -> None:
        """
        Initialize a LockdownObject object.

        :param str id: Lockdown rule identifier.
        :param bool paused: Whether this zone lockdown is currently paused.
        :param str description: A note that you can use to describe the reason for
               a Lockdown rule.
        :param List[str] urls: URLs to be included in this rule definition.
               Wildcards are permitted. The URL pattern entered here will be escaped
               before use. This limits the URL to just simple wildcard patterns.
        :param List[LockdownObjectConfigurationsItem] configurations: List of IP
               addresses or CIDR ranges to use for this rule. This can include any number
               of ip or ip_range configurations that can access the provided URLs.
        """
        self.id = id
        self.paused = paused
        self.description = description
        self.urls = urls
        self.configurations = configurations

    @classmethod
    def from_dict(cls, _dict: Dict) -> 'LockdownObject':
        """Initialize a LockdownObject object from a json dictionary."""
        args = {}
        if 'id' in _dict:
            args['id'] = _dict.get('id')
        else:
            raise ValueError('Required property \'id\' not present in LockdownObject JSON')
        if 'paused' in _dict:
            args['paused'] = _dict.get('paused')
        else:
            raise ValueError('Required property \'paused\' not present in LockdownObject JSON')
        if 'description' in _dict:
            args['description'] = _dict.get('description')
        else:
            raise ValueError('Required property \'description\' not present in LockdownObject JSON')
        if 'urls' in _dict:
            args['urls'] = _dict.get('urls')
        else:
            raise ValueError('Required property \'urls\' not present in LockdownObject JSON')
        if 'configurations' in _dict:
            args['configurations'] = [LockdownObjectConfigurationsItem.from_dict(x) for x in _dict.get('configurations')]
        else:
            raise ValueError('Required property \'configurations\' not present in LockdownObject JSON')
        return cls(**args)

    @classmethod
    def _from_dict(cls, _dict):
        """Initialize a LockdownObject object from a json dictionary."""
        return cls.from_dict(_dict)

    def to_dict(self) -> Dict:
        """Return a json dictionary representing this model."""
        _dict = {}
        if hasattr(self, 'id') and self.id is not None:
            _dict['id'] = self.id
        if hasattr(self, 'paused') and self.paused is not None:
            _dict['paused'] = self.paused
        if hasattr(self, 'description') and self.description is not None:
            _dict['description'] = self.description
        if hasattr(self, 'urls') and self.urls is not None:
            _dict['urls'] = self.urls
        if hasattr(self, 'configurations') and self.configurations is not None:
            _dict['configurations'] = [x.to_dict() for x in self.configurations]
        return _dict

    def _to_dict(self):
        """Return a json dictionary representing this model."""
        return self.to_dict()

    def __str__(self) -> str:
        """Return a `str` version of this LockdownObject object."""
        return json.dumps(self.to_dict(), indent=2)

    def __eq__(self, other: 'LockdownObject') -> bool:
        """Return `true` when self and other are equal, false otherwise."""
        if not isinstance(other, self.__class__):
            return False
        return self.__dict__ == other.__dict__

    def __ne__(self, other: 'LockdownObject') -> bool:
        """Return `true` when self and other are not equal, false otherwise."""
        return not self == other

class LockdownResp():
    """
    lockdown response.

    :attr bool success: Was operation successful.
    :attr List[List[str]] errors: Array of errors encountered.
    :attr List[List[str]] messages: Array of messages returned.
    :attr LockdownObject result: lockdown object.
    """

    def __init__(self,
                 success: bool,
                 errors: List[List[str]],
                 messages: List[List[str]],
                 result: 'LockdownObject') -> None:
        """
        Initialize a LockdownResp object.

        :param bool success: Was operation successful.
        :param List[List[str]] errors: Array of errors encountered.
        :param List[List[str]] messages: Array of messages returned.
        :param LockdownObject result: lockdown object.
        """
        self.success = success
        self.errors = errors
        self.messages = messages
        self.result = result

    @classmethod
    def from_dict(cls, _dict: Dict) -> 'LockdownResp':
        """Initialize a LockdownResp object from a json dictionary."""
        args = {}
        if 'success' in _dict:
            args['success'] = _dict.get('success')
        else:
            raise ValueError('Required property \'success\' not present in LockdownResp JSON')
        if 'errors' in _dict:
            args['errors'] = _dict.get('errors')
        else:
            raise ValueError('Required property \'errors\' not present in LockdownResp JSON')
        if 'messages' in _dict:
            args['messages'] = _dict.get('messages')
        else:
            raise ValueError('Required property \'messages\' not present in LockdownResp JSON')
        if 'result' in _dict:
            args['result'] = LockdownObject.from_dict(_dict.get('result'))
        else:
            raise ValueError('Required property \'result\' not present in LockdownResp JSON')
        return cls(**args)

    @classmethod
    def _from_dict(cls, _dict):
        """Initialize a LockdownResp object from a json dictionary."""
        return cls.from_dict(_dict)

    def to_dict(self) -> Dict:
        """Return a json dictionary representing this model."""
        _dict = {}
        if hasattr(self, 'success') and self.success is not None:
            _dict['success'] = self.success
        if hasattr(self, 'errors') and self.errors is not None:
            _dict['errors'] = self.errors
        if hasattr(self, 'messages') and self.messages is not None:
            _dict['messages'] = self.messages
        if hasattr(self, 'result') and self.result is not None:
            _dict['result'] = self.result.to_dict()
        return _dict

    def _to_dict(self):
        """Return a json dictionary representing this model."""
        return self.to_dict()

    def __str__(self) -> str:
        """Return a `str` version of this LockdownResp object."""
        return json.dumps(self.to_dict(), indent=2)

    def __eq__(self, other: 'LockdownResp') -> bool:
        """Return `true` when self and other are equal, false otherwise."""
        if not isinstance(other, self.__class__):
            return False
        return self.__dict__ == other.__dict__

    def __ne__(self, other: 'LockdownResp') -> bool:
        """Return `true` when self and other are not equal, false otherwise."""
        return not self == other



# coding: utf-8

# (C) Copyright IBM Corp. 2020.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""
Range Applications
"""

from datetime import datetime
from enum import Enum
from typing import Dict, List
import json

from ibm_cloud_sdk_core import BaseService, DetailedResponse
from ibm_cloud_sdk_core.authenticators.authenticator import Authenticator
from ibm_cloud_sdk_core.get_authenticator import get_authenticator_from_environment
from ibm_cloud_sdk_core.utils import convert_model, datetime_to_string, string_to_datetime

from .common import get_sdk_headers

##############################################################################
# Service
##############################################################################

class RangeApplicationsV1(BaseService):
    """The Range Applications V1 service."""

    DEFAULT_SERVICE_URL = 'https://api.cis.cloud.ibm.com'
    DEFAULT_SERVICE_NAME = 'range_applications'

    @classmethod
    def new_instance(cls,
                     crn: str,
                     zone_identifier: str,
                     service_name: str = DEFAULT_SERVICE_NAME,
                    ) -> 'RangeApplicationsV1':
        """
        Return a new client for the Range Applications service using the specified
               parameters and external configuration.

        :param str crn: Full url-encoded cloud resource name (CRN) of resource
               instance.

        :param str zone_identifier: zone identifier.
        """
        if crn is None:
            raise ValueError('crn must be provided')
        if zone_identifier is None:
            raise ValueError('zone_identifier must be provided')

        authenticator = get_authenticator_from_environment(service_name)
        service = cls(
            crn,
            zone_identifier,
            authenticator
            )
        service.configure_service(service_name)
        return service

    def __init__(self,
                 crn: str,
                 zone_identifier: str,
                 authenticator: Authenticator = None,
                ) -> None:
        """
        Construct a new client for the Range Applications service.

        :param str crn: Full url-encoded cloud resource name (CRN) of resource
               instance.

        :param str zone_identifier: zone identifier.

        :param Authenticator authenticator: The authenticator specifies the authentication mechanism.
               Get up to date information from https://github.com/IBM/python-sdk-core/blob/master/README.md
               about initializing the authenticator of your choice.
        """
        if crn is None:
            raise ValueError('crn must be provided')
        if zone_identifier is None:
            raise ValueError('zone_identifier must be provided')

        BaseService.__init__(self,
                             service_url=self.DEFAULT_SERVICE_URL,
                             authenticator=authenticator)
        self.crn = crn
        self.zone_identifier = zone_identifier


    #########################
    # Range Applications
    #########################


    def list_range_apps(self,
        *,
        page: int = None,
        per_page: int = None,
        order: str = None,
        direction: str = None,
        **kwargs
    ) -> DetailedResponse:
        """
        List range applications.

        Get a list of currently existing Range Applications inside a zone.

        :param int page: (optional) Page number of paginated results.
        :param int per_page: (optional) Maximum number of Range applications per
               page.
        :param str order: (optional) Field by which to order the list of Range
               applications.
        :param str direction: (optional) Direction in which to order results
               [ascending/descending order].
        :param dict headers: A `dict` containing the request headers
        :return: A `DetailedResponse` containing the result, headers and HTTP status code.
        :rtype: DetailedResponse with `dict` result representing a `RangeApplications` object
        """

        headers = {}
        sdk_headers = get_sdk_headers(service_name=self.DEFAULT_SERVICE_NAME,
                                      service_version='V1',
                                      operation_id='list_range_apps')
        headers.update(sdk_headers)

        params = {
            'page': page,
            'per_page': per_page,
            'order': order,
            'direction': direction
        }

        if 'headers' in kwargs:
            headers.update(kwargs.get('headers'))

        url = '/v1/{0}/zones/{1}/range/apps'.format(
            *self.encode_path_vars(self.crn, self.zone_identifier))
        request = self.prepare_request(method='GET',
                                       url=url,
                                       headers=headers,
                                       params=params)

        response = self.send(request)
        return response


    def create_range_app(self,
        protocol: str,
        dns: 'RangeAppReqDns',
        *,
        origin_direct: List[str] = None,
        origin_dns: 'RangeAppReqOriginDns' = None,
        origin_port: int = None,
        ip_firewall: bool = None,
        proxy_protocol: str = None,
        edge_ips: 'RangeAppReqEdgeIps' = None,
        traffic_type: str = None,
        tls: str = None,
        **kwargs
    ) -> DetailedResponse:
        """
        Create Range Application.

        Create a Range Applications inside a zone.

        :param str protocol: Defines the protocol and port for this application.
        :param RangeAppReqDns dns: Name and type of the DNS record for this
               application.
        :param List[str] origin_direct: (optional) IP address and port of the
               origin for this Range application. If configuring a load balancer, use
               'origin_dns' and 'origin_port'. This can not be combined with 'origin_dns'
               and 'origin_port'.
        :param RangeAppReqOriginDns origin_dns: (optional) DNS record pointing to
               the origin for this Range application. This is used for configuring a load
               balancer. When specifying an individual IP address, use 'origin_direct'.
               This requires 'origin_port' and can not be combined with 'origin_direct'.
        :param int origin_port: (optional) Port at the origin that listens to
               traffic from this Range application. Requires 'origin_dns' and can not be
               combined with 'origin_direct'.
        :param bool ip_firewall: (optional) Enables the IP Firewall for this
               application. Only available for TCP applications.
        :param str proxy_protocol: (optional) Allows for the true client IP to be
               passed to the service.
        :param RangeAppReqEdgeIps edge_ips: (optional) Configures IP version for
               the hostname of this application. Default is {"type":"dynamic",
               "connectivity":"all"}.
        :param str traffic_type: (optional) Configure how traffic is handled at the
               edge. If set to "direct" traffic is passed through to the service. In the
               case of "http" or "https" HTTP/s features at the edge are applied ot this
               traffic.
        :param str tls: (optional) Configure if and how TLS connections are
               terminated at the edge.
        :param dict headers: A `dict` containing the request headers
        :return: A `DetailedResponse` containing the result, headers and HTTP status code.
        :rtype: DetailedResponse with `dict` result representing a `RangeApplicationResp` object
        """

        if protocol is None:
            raise ValueError('protocol must be provided')
        if dns is None:
            raise ValueError('dns must be provided')
        dns = convert_model(dns)
        if origin_dns is not None:
            origin_dns = convert_model(origin_dns)
        if edge_ips is not None:
            edge_ips = convert_model(edge_ips)
        headers = {}
        sdk_headers = get_sdk_headers(service_name=self.DEFAULT_SERVICE_NAME,
                                      service_version='V1',
                                      operation_id='create_range_app')
        headers.update(sdk_headers)

        data = {
            'protocol': protocol,
            'dns': dns,
            'origin_direct': origin_direct,
            'origin_dns': origin_dns,
            'origin_port': origin_port,
            'ip_firewall': ip_firewall,
            'proxy_protocol': proxy_protocol,
            'edge_ips': edge_ips,
            'traffic_type': traffic_type,
            'tls': tls
        }
        data = {k: v for (k, v) in data.items() if v is not None}
        data = json.dumps(data)
        headers['content-type'] = 'application/json'

        if 'headers' in kwargs:
            headers.update(kwargs.get('headers'))

        url = '/v1/{0}/zones/{1}/range/apps'.format(
            *self.encode_path_vars(self.crn, self.zone_identifier))
        request = self.prepare_request(method='POST',
                                       url=url,
                                       headers=headers,
                                       data=data)

        response = self.send(request)
        return response


    def get_range_app(self,
        app_identifier: str,
        **kwargs
    ) -> DetailedResponse:
        """
        Get range application a zone.

        Get the application configuration of a specific application inside a zone.

        :param str app_identifier: application identifier.
        :param dict headers: A `dict` containing the request headers
        :return: A `DetailedResponse` containing the result, headers and HTTP status code.
        :rtype: DetailedResponse with `dict` result representing a `RangeApplicationResp` object
        """

        if app_identifier is None:
            raise ValueError('app_identifier must be provided')
        headers = {}
        sdk_headers = get_sdk_headers(service_name=self.DEFAULT_SERVICE_NAME,
                                      service_version='V1',
                                      operation_id='get_range_app')
        headers.update(sdk_headers)

        if 'headers' in kwargs:
            headers.update(kwargs.get('headers'))

        url = '/v1/{0}/zones/{1}/range/apps/{2}'.format(
            *self.encode_path_vars(self.crn, self.zone_identifier, app_identifier))
        request = self.prepare_request(method='GET',
                                       url=url,
                                       headers=headers)

        response = self.send(request)
        return response


    def update_range_app(self,
        app_identifier: str,
        protocol: str,
        dns: 'RangeAppReqDns',
        *,
        origin_direct: List[str] = None,
        origin_dns: 'RangeAppReqOriginDns' = None,
        origin_port: int = None,
        ip_firewall: bool = None,
        proxy_protocol: str = None,
        edge_ips: 'RangeAppReqEdgeIps' = None,
        traffic_type: str = None,
        tls: str = None,
        **kwargs
    ) -> DetailedResponse:
        """
        Update range application.

        Update a Range Application inside a zone.

        :param str app_identifier: application identifier.
        :param str protocol: Defines the protocol and port for this application.
        :param RangeAppReqDns dns: Name and type of the DNS record for this
               application.
        :param List[str] origin_direct: (optional) IP address and port of the
               origin for this Range application. If configuring a load balancer, use
               'origin_dns' and 'origin_port'. This can not be combined with 'origin_dns'
               and 'origin_port'.
        :param RangeAppReqOriginDns origin_dns: (optional) DNS record pointing to
               the origin for this Range application. This is used for configuring a load
               balancer. When specifying an individual IP address, use 'origin_direct'.
               This requires 'origin_port' and can not be combined with 'origin_direct'.
        :param int origin_port: (optional) Port at the origin that listens to
               traffic from this Range application. Requires 'origin_dns' and can not be
               combined with 'origin_direct'.
        :param bool ip_firewall: (optional) Enables the IP Firewall for this
               application. Only available for TCP applications.
        :param str proxy_protocol: (optional) Allows for the true client IP to be
               passed to the service.
        :param RangeAppReqEdgeIps edge_ips: (optional) Configures IP version for
               the hostname of this application. Default is {"type":"dynamic",
               "connectivity":"all"}.
        :param str traffic_type: (optional) Configure how traffic is handled at the
               edge. If set to "direct" traffic is passed through to the service. In the
               case of "http" or "https" HTTP/s features at the edge are applied ot this
               traffic.
        :param str tls: (optional) Configure if and how TLS connections are
               terminated at the edge.
        :param dict headers: A `dict` containing the request headers
        :return: A `DetailedResponse` containing the result, headers and HTTP status code.
        :rtype: DetailedResponse with `dict` result representing a `RangeApplicationResp` object
        """

        if app_identifier is None:
            raise ValueError('app_identifier must be provided')
        if protocol is None:
            raise ValueError('protocol must be provided')
        if dns is None:
            raise ValueError('dns must be provided')
        dns = convert_model(dns)
        if origin_dns is not None:
            origin_dns = convert_model(origin_dns)
        if edge_ips is not None:
            edge_ips = convert_model(edge_ips)
        headers = {}
        sdk_headers = get_sdk_headers(service_name=self.DEFAULT_SERVICE_NAME,
                                      service_version='V1',
                                      operation_id='update_range_app')
        headers.update(sdk_headers)

        data = {
            'protocol': protocol,
            'dns': dns,
            'origin_direct': origin_direct,
            'origin_dns': origin_dns,
            'origin_port': origin_port,
            'ip_firewall': ip_firewall,
            'proxy_protocol': proxy_protocol,
            'edge_ips': edge_ips,
            'traffic_type': traffic_type,
            'tls': tls
        }
        data = {k: v for (k, v) in data.items() if v is not None}
        data = json.dumps(data)
        headers['content-type'] = 'application/json'

        if 'headers' in kwargs:
            headers.update(kwargs.get('headers'))

        url = '/v1/{0}/zones/{1}/range/apps/{2}'.format(
            *self.encode_path_vars(self.crn, self.zone_identifier, app_identifier))
        request = self.prepare_request(method='PUT',
                                       url=url,
                                       headers=headers,
                                       data=data)

        response = self.send(request)
        return response


    def delete_range_app(self,
        app_identifier: str,
        **kwargs
    ) -> DetailedResponse:
        """
        Delete range application.

        Delete a specific application configuration.

        :param str app_identifier: application identifier.
        :param dict headers: A `dict` containing the request headers
        :return: A `DetailedResponse` containing the result, headers and HTTP status code.
        :rtype: DetailedResponse with `dict` result representing a `RangeApplicationResp` object
        """

        if app_identifier is None:
            raise ValueError('app_identifier must be provided')
        headers = {}
        sdk_headers = get_sdk_headers(service_name=self.DEFAULT_SERVICE_NAME,
                                      service_version='V1',
                                      operation_id='delete_range_app')
        headers.update(sdk_headers)

        if 'headers' in kwargs:
            headers.update(kwargs.get('headers'))

        url = '/v1/{0}/zones/{1}/range/apps/{2}'.format(
            *self.encode_path_vars(self.crn, self.zone_identifier, app_identifier))
        request = self.prepare_request(method='DELETE',
                                       url=url,
                                       headers=headers)

        response = self.send(request)
        return response


class ListRangeAppsEnums:
    """
    Enums for list_range_apps parameters.
    """

    class Order(str, Enum):
        """
        Field by which to order the list of Range applications.
        """
        PROTOCOL = 'protocol'
        APP_ID = 'app_id'
        CREATED_ON = 'created_on'
        MODIFIED_ON = 'modified_on'
        DNS = 'dns'
    class Direction(str, Enum):
        """
        Direction in which to order results [ascending/descending order].
        """
        ASC = 'asc'
        DESC = 'desc'


##############################################################################
# Models
##############################################################################


class RangeAppReqDns():
    """
    Name and type of the DNS record for this application.

    :attr str type: (optional) DNS record type.
    :attr str name: (optional) DNS record name.
    """

    def __init__(self,
                 *,
                 type: str = None,
                 name: str = None) -> None:
        """
        Initialize a RangeAppReqDns object.

        :param str type: (optional) DNS record type.
        :param str name: (optional) DNS record name.
        """
        self.type = type
        self.name = name

    @classmethod
    def from_dict(cls, _dict: Dict) -> 'RangeAppReqDns':
        """Initialize a RangeAppReqDns object from a json dictionary."""
        args = {}
        if 'type' in _dict:
            args['type'] = _dict.get('type')
        if 'name' in _dict:
            args['name'] = _dict.get('name')
        return cls(**args)

    @classmethod
    def _from_dict(cls, _dict):
        """Initialize a RangeAppReqDns object from a json dictionary."""
        return cls.from_dict(_dict)

    def to_dict(self) -> Dict:
        """Return a json dictionary representing this model."""
        _dict = {}
        if hasattr(self, 'type') and self.type is not None:
            _dict['type'] = self.type
        if hasattr(self, 'name') and self.name is not None:
            _dict['name'] = self.name
        return _dict

    def _to_dict(self):
        """Return a json dictionary representing this model."""
        return self.to_dict()

    def __str__(self) -> str:
        """Return a `str` version of this RangeAppReqDns object."""
        return json.dumps(self.to_dict(), indent=2)

    def __eq__(self, other: 'RangeAppReqDns') -> bool:
        """Return `true` when self and other are equal, false otherwise."""
        if not isinstance(other, self.__class__):
            return False
        return self.__dict__ == other.__dict__

    def __ne__(self, other: 'RangeAppReqDns') -> bool:
        """Return `true` when self and other are not equal, false otherwise."""
        return not self == other

class RangeAppReqEdgeIps():
    """
    Configures IP version for the hostname of this application. Default is
    {"type":"dynamic", "connectivity":"all"}.

    :attr str type: (optional) The type of edge IP configuration.
    :attr str connectivity: (optional) Specifies the IP version (or all).
    """

    def __init__(self,
                 *,
                 type: str = None,
                 connectivity: str = None) -> None:
        """
        Initialize a RangeAppReqEdgeIps object.

        :param str type: (optional) The type of edge IP configuration.
        :param str connectivity: (optional) Specifies the IP version (or all).
        """
        self.type = type
        self.connectivity = connectivity

    @classmethod
    def from_dict(cls, _dict: Dict) -> 'RangeAppReqEdgeIps':
        """Initialize a RangeAppReqEdgeIps object from a json dictionary."""
        args = {}
        if 'type' in _dict:
            args['type'] = _dict.get('type')
        if 'connectivity' in _dict:
            args['connectivity'] = _dict.get('connectivity')
        return cls(**args)

    @classmethod
    def _from_dict(cls, _dict):
        """Initialize a RangeAppReqEdgeIps object from a json dictionary."""
        return cls.from_dict(_dict)

    def to_dict(self) -> Dict:
        """Return a json dictionary representing this model."""
        _dict = {}
        if hasattr(self, 'type') and self.type is not None:
            _dict['type'] = self.type
        if hasattr(self, 'connectivity') and self.connectivity is not None:
            _dict['connectivity'] = self.connectivity
        return _dict

    def _to_dict(self):
        """Return a json dictionary representing this model."""
        return self.to_dict()

    def __str__(self) -> str:
        """Return a `str` version of this RangeAppReqEdgeIps object."""
        return json.dumps(self.to_dict(), indent=2)

    def __eq__(self, other: 'RangeAppReqEdgeIps') -> bool:
        """Return `true` when self and other are equal, false otherwise."""
        if not isinstance(other, self.__class__):
            return False
        return self.__dict__ == other.__dict__

    def __ne__(self, other: 'RangeAppReqEdgeIps') -> bool:
        """Return `true` when self and other are not equal, false otherwise."""
        return not self == other

    class TypeEnum(str, Enum):
        """
        The type of edge IP configuration.
        """
        DYNAMIC = 'dynamic'


    class ConnectivityEnum(str, Enum):
        """
        Specifies the IP version (or all).
        """
        IPV4 = 'ipv4'
        IPV6 = 'ipv6'
        ALL = 'all'


class RangeAppReqOriginDns():
    """
    DNS record pointing to the origin for this Range application. This is used for
    configuring a load balancer. When specifying an individual IP address, use
    'origin_direct'. This requires 'origin_port' and can not be combined with
    'origin_direct'.

    :attr str name: Name of the origin.
    """

    def __init__(self,
                 name: str) -> None:
        """
        Initialize a RangeAppReqOriginDns object.

        :param str name: Name of the origin.
        """
        self.name = name

    @classmethod
    def from_dict(cls, _dict: Dict) -> 'RangeAppReqOriginDns':
        """Initialize a RangeAppReqOriginDns object from a json dictionary."""
        args = {}
        if 'name' in _dict:
            args['name'] = _dict.get('name')
        else:
            raise ValueError('Required property \'name\' not present in RangeAppReqOriginDns JSON')
        return cls(**args)

    @classmethod
    def _from_dict(cls, _dict):
        """Initialize a RangeAppReqOriginDns object from a json dictionary."""
        return cls.from_dict(_dict)

    def to_dict(self) -> Dict:
        """Return a json dictionary representing this model."""
        _dict = {}
        if hasattr(self, 'name') and self.name is not None:
            _dict['name'] = self.name
        return _dict

    def _to_dict(self):
        """Return a json dictionary representing this model."""
        return self.to_dict()

    def __str__(self) -> str:
        """Return a `str` version of this RangeAppReqOriginDns object."""
        return json.dumps(self.to_dict(), indent=2)

    def __eq__(self, other: 'RangeAppReqOriginDns') -> bool:
        """Return `true` when self and other are equal, false otherwise."""
        if not isinstance(other, self.__class__):
            return False
        return self.__dict__ == other.__dict__

    def __ne__(self, other: 'RangeAppReqOriginDns') -> bool:
        """Return `true` when self and other are not equal, false otherwise."""
        return not self == other

class RangeApplicationObjectDns():
    """
    The name and type of DNS record for the Range application.

    :attr str type: (optional) The type of DNS record associated with the
          application.
    :attr str name: (optional) The name of the DNS record associated with the
          application.
    """

    def __init__(self,
                 *,
                 type: str = None,
                 name: str = None) -> None:
        """
        Initialize a RangeApplicationObjectDns object.

        :param str type: (optional) The type of DNS record associated with the
               application.
        :param str name: (optional) The name of the DNS record associated with the
               application.
        """
        self.type = type
        self.name = name

    @classmethod
    def from_dict(cls, _dict: Dict) -> 'RangeApplicationObjectDns':
        """Initialize a RangeApplicationObjectDns object from a json dictionary."""
        args = {}
        if 'type' in _dict:
            args['type'] = _dict.get('type')
        if 'name' in _dict:
            args['name'] = _dict.get('name')
        return cls(**args)

    @classmethod
    def _from_dict(cls, _dict):
        """Initialize a RangeApplicationObjectDns object from a json dictionary."""
        return cls.from_dict(_dict)

    def to_dict(self) -> Dict:
        """Return a json dictionary representing this model."""
        _dict = {}
        if hasattr(self, 'type') and self.type is not None:
            _dict['type'] = self.type
        if hasattr(self, 'name') and self.name is not None:
            _dict['name'] = self.name
        return _dict

    def _to_dict(self):
        """Return a json dictionary representing this model."""
        return self.to_dict()

    def __str__(self) -> str:
        """Return a `str` version of this RangeApplicationObjectDns object."""
        return json.dumps(self.to_dict(), indent=2)

    def __eq__(self, other: 'RangeApplicationObjectDns') -> bool:
        """Return `true` when self and other are equal, false otherwise."""
        if not isinstance(other, self.__class__):
            return False
        return self.__dict__ == other.__dict__

    def __ne__(self, other: 'RangeApplicationObjectDns') -> bool:
        """Return `true` when self and other are not equal, false otherwise."""
        return not self == other

    class TypeEnum(str, Enum):
        """
        The type of DNS record associated with the application.
        """
        CNAME = 'CNAME'


class RangeApplicationObjectEdgeIps():
    """
    Configures IP version for the hostname of this application.

    :attr str type: (optional) The type of edge IP configuration.
    :attr str connectivity: (optional) Specifies the IP version (or all).
    """

    def __init__(self,
                 *,
                 type: str = None,
                 connectivity: str = None) -> None:
        """
        Initialize a RangeApplicationObjectEdgeIps object.

        :param str type: (optional) The type of edge IP configuration.
        :param str connectivity: (optional) Specifies the IP version (or all).
        """
        self.type = type
        self.connectivity = connectivity

    @classmethod
    def from_dict(cls, _dict: Dict) -> 'RangeApplicationObjectEdgeIps':
        """Initialize a RangeApplicationObjectEdgeIps object from a json dictionary."""
        args = {}
        if 'type' in _dict:
            args['type'] = _dict.get('type')
        if 'connectivity' in _dict:
            args['connectivity'] = _dict.get('connectivity')
        return cls(**args)

    @classmethod
    def _from_dict(cls, _dict):
        """Initialize a RangeApplicationObjectEdgeIps object from a json dictionary."""
        return cls.from_dict(_dict)

    def to_dict(self) -> Dict:
        """Return a json dictionary representing this model."""
        _dict = {}
        if hasattr(self, 'type') and self.type is not None:
            _dict['type'] = self.type
        if hasattr(self, 'connectivity') and self.connectivity is not None:
            _dict['connectivity'] = self.connectivity
        return _dict

    def _to_dict(self):
        """Return a json dictionary representing this model."""
        return self.to_dict()

    def __str__(self) -> str:
        """Return a `str` version of this RangeApplicationObjectEdgeIps object."""
        return json.dumps(self.to_dict(), indent=2)

    def __eq__(self, other: 'RangeApplicationObjectEdgeIps') -> bool:
        """Return `true` when self and other are equal, false otherwise."""
        if not isinstance(other, self.__class__):
            return False
        return self.__dict__ == other.__dict__

    def __ne__(self, other: 'RangeApplicationObjectEdgeIps') -> bool:
        """Return `true` when self and other are not equal, false otherwise."""
        return not self == other

    class TypeEnum(str, Enum):
        """
        The type of edge IP configuration.
        """
        DYNAMIC = 'dynamic'


    class ConnectivityEnum(str, Enum):
        """
        Specifies the IP version (or all).
        """
        IPV4 = 'ipv4'
        IPV6 = 'ipv6'
        ALL = 'all'


class RangeApplicationObject():
    """
    range application object.

    :attr str id: (optional) Application identifier.
    :attr str protocol: (optional) Port configuration.
    :attr RangeApplicationObjectDns dns: (optional) The name and type of DNS record
          for the Range application.
    :attr List[str] origin_direct: (optional) A list of destination addresses to the
          origin.
    :attr bool ip_firewall: (optional) Enables the IP Firewall for this application.
    :attr str proxy_protocol: (optional) Allows for the true client IP to be passed
          to the service.
    :attr RangeApplicationObjectEdgeIps edge_ips: (optional) Configures IP version
          for the hostname of this application.
    :attr str tls: (optional) Specifies the TLS termination at the edge.
    :attr str traffic_type: (optional) Configure how traffic is handled at the edge.
          If set to "direct" traffic is passed through to the service. In the case of
          "http" or "https" HTTP/s features at the edge are applied ot this traffic.
    :attr datetime created_on: (optional) When the Application was created.
    :attr datetime modified_on: (optional) When the Application was last modified.
    """

    def __init__(self,
                 *,
                 id: str = None,
                 protocol: str = None,
                 dns: 'RangeApplicationObjectDns' = None,
                 origin_direct: List[str] = None,
                 ip_firewall: bool = None,
                 proxy_protocol: str = None,
                 edge_ips: 'RangeApplicationObjectEdgeIps' = None,
                 tls: str = None,
                 traffic_type: str = None,
                 created_on: datetime = None,
                 modified_on: datetime = None) -> None:
        """
        Initialize a RangeApplicationObject object.

        :param str id: (optional) Application identifier.
        :param str protocol: (optional) Port configuration.
        :param RangeApplicationObjectDns dns: (optional) The name and type of DNS
               record for the Range application.
        :param List[str] origin_direct: (optional) A list of destination addresses
               to the origin.
        :param bool ip_firewall: (optional) Enables the IP Firewall for this
               application.
        :param str proxy_protocol: (optional) Allows for the true client IP to be
               passed to the service.
        :param RangeApplicationObjectEdgeIps edge_ips: (optional) Configures IP
               version for the hostname of this application.
        :param str tls: (optional) Specifies the TLS termination at the edge.
        :param str traffic_type: (optional) Configure how traffic is handled at the
               edge. If set to "direct" traffic is passed through to the service. In the
               case of "http" or "https" HTTP/s features at the edge are applied ot this
               traffic.
        :param datetime created_on: (optional) When the Application was created.
        :param datetime modified_on: (optional) When the Application was last
               modified.
        """
        self.id = id
        self.protocol = protocol
        self.dns = dns
        self.origin_direct = origin_direct
        self.ip_firewall = ip_firewall
        self.proxy_protocol = proxy_protocol
        self.edge_ips = edge_ips
        self.tls = tls
        self.traffic_type = traffic_type
        self.created_on = created_on
        self.modified_on = modified_on

    @classmethod
    def from_dict(cls, _dict: Dict) -> 'RangeApplicationObject':
        """Initialize a RangeApplicationObject object from a json dictionary."""
        args = {}
        if 'id' in _dict:
            args['id'] = _dict.get('id')
        if 'protocol' in _dict:
            args['protocol'] = _dict.get('protocol')
        if 'dns' in _dict:
            args['dns'] = RangeApplicationObjectDns.from_dict(_dict.get('dns'))
        if 'origin_direct' in _dict:
            args['origin_direct'] = _dict.get('origin_direct')
        if 'ip_firewall' in _dict:
            args['ip_firewall'] = _dict.get('ip_firewall')
        if 'proxy_protocol' in _dict:
            args['proxy_protocol'] = _dict.get('proxy_protocol')
        if 'edge_ips' in _dict:
            args['edge_ips'] = RangeApplicationObjectEdgeIps.from_dict(_dict.get('edge_ips'))
        if 'tls' in _dict:
            args['tls'] = _dict.get('tls')
        if 'traffic_type' in _dict:
            args['traffic_type'] = _dict.get('traffic_type')
        if 'created_on' in _dict:
            args['created_on'] = string_to_datetime(_dict.get('created_on'))
        if 'modified_on' in _dict:
            args['modified_on'] = string_to_datetime(_dict.get('modified_on'))
        return cls(**args)

    @classmethod
    def _from_dict(cls, _dict):
        """Initialize a RangeApplicationObject object from a json dictionary."""
        return cls.from_dict(_dict)

    def to_dict(self) -> Dict:
        """Return a json dictionary representing this model."""
        _dict = {}
        if hasattr(self, 'id') and self.id is not None:
            _dict['id'] = self.id
        if hasattr(self, 'protocol') and self.protocol is not None:
            _dict['protocol'] = self.protocol
        if hasattr(self, 'dns') and self.dns is not None:
            _dict['dns'] = self.dns.to_dict()
        if hasattr(self, 'origin_direct') and self.origin_direct is not None:
            _dict['origin_direct'] = self.origin_direct
        if hasattr(self, 'ip_firewall') and self.ip_firewall is not None:
            _dict['ip_firewall'] = self.ip_firewall
        if hasattr(self, 'proxy_protocol') and self.proxy_protocol is not None:
            _dict['proxy_protocol'] = self.proxy_protocol
        if hasattr(self, 'edge_ips') and self.edge_ips is not None:
            _dict['edge_ips'] = self.edge_ips.to_dict()
        if hasattr(self, 'tls') and self.tls is not None:
            _dict['tls'] = self.tls
        if hasattr(self, 'traffic_type') and self.traffic_type is not None:
            _dict['traffic_type'] = self.traffic_type
        if hasattr(self, 'created_on') and self.created_on is not None:
            _dict['created_on'] = datetime_to_string(self.created_on)
        if hasattr(self, 'modified_on') and self.modified_on is not None:
            _dict['modified_on'] = datetime_to_string(self.modified_on)
        return _dict

    def _to_dict(self):
        """Return a json dictionary representing this model."""
        return self.to_dict()

    def __str__(self) -> str:
        """Return a `str` version of this RangeApplicationObject object."""
        return json.dumps(self.to_dict(), indent=2)

    def __eq__(self, other: 'RangeApplicationObject') -> bool:
        """Return `true` when self and other are equal, false otherwise."""
        if not isinstance(other, self.__class__):
            return False
        return self.__dict__ == other.__dict__

    def __ne__(self, other: 'RangeApplicationObject') -> bool:
        """Return `true` when self and other are not equal, false otherwise."""
        return not self == other

    class ProxyProtocolEnum(str, Enum):
        """
        Allows for the true client IP to be passed to the service.
        """
        OFF = 'off'
        V1 = 'v1'
        V2 = 'v2'
        SIMPLE = 'simple'


    class TlsEnum(str, Enum):
        """
        Specifies the TLS termination at the edge.
        """
        OFF = 'off'
        FLEXIBLE = 'flexible'
        FULL = 'full'
        STRICT = 'strict'


    class TrafficTypeEnum(str, Enum):
        """
        Configure how traffic is handled at the edge. If set to "direct" traffic is passed
        through to the service. In the case of "http" or "https" HTTP/s features at the
        edge are applied ot this traffic.
        """
        DIRECT = 'direct'
        HTTP = 'http'
        HTTPS = 'https'


class RangeApplicationResp():
    """
    range application response.

    :attr bool success: Was the get successful.
    :attr List[List[str]] errors: errors.
    :attr List[List[str]] messages: messages.
    :attr RangeApplicationObject result: range application object.
    """

    def __init__(self,
                 success: bool,
                 errors: List[List[str]],
                 messages: List[List[str]],
                 result: 'RangeApplicationObject') -> None:
        """
        Initialize a RangeApplicationResp object.

        :param bool success: Was the get successful.
        :param List[List[str]] errors: errors.
        :param List[List[str]] messages: messages.
        :param RangeApplicationObject result: range application object.
        """
        self.success = success
        self.errors = errors
        self.messages = messages
        self.result = result

    @classmethod
    def from_dict(cls, _dict: Dict) -> 'RangeApplicationResp':
        """Initialize a RangeApplicationResp object from a json dictionary."""
        args = {}
        if 'success' in _dict:
            args['success'] = _dict.get('success')
        else:
            raise ValueError('Required property \'success\' not present in RangeApplicationResp JSON')
        if 'errors' in _dict:
            args['errors'] = _dict.get('errors')
        else:
            raise ValueError('Required property \'errors\' not present in RangeApplicationResp JSON')
        if 'messages' in _dict:
            args['messages'] = _dict.get('messages')
        else:
            raise ValueError('Required property \'messages\' not present in RangeApplicationResp JSON')
        if 'result' in _dict:
            args['result'] = RangeApplicationObject.from_dict(_dict.get('result'))
        else:
            raise ValueError('Required property \'result\' not present in RangeApplicationResp JSON')
        return cls(**args)

    @classmethod
    def _from_dict(cls, _dict):
        """Initialize a RangeApplicationResp object from a json dictionary."""
        return cls.from_dict(_dict)

    def to_dict(self) -> Dict:
        """Return a json dictionary representing this model."""
        _dict = {}
        if hasattr(self, 'success') and self.success is not None:
            _dict['success'] = self.success
        if hasattr(self, 'errors') and self.errors is not None:
            _dict['errors'] = self.errors
        if hasattr(self, 'messages') and self.messages is not None:
            _dict['messages'] = self.messages
        if hasattr(self, 'result') and self.result is not None:
            _dict['result'] = self.result.to_dict()
        return _dict

    def _to_dict(self):
        """Return a json dictionary representing this model."""
        return self.to_dict()

    def __str__(self) -> str:
        """Return a `str` version of this RangeApplicationResp object."""
        return json.dumps(self.to_dict(), indent=2)

    def __eq__(self, other: 'RangeApplicationResp') -> bool:
        """Return `true` when self and other are equal, false otherwise."""
        if not isinstance(other, self.__class__):
            return False
        return self.__dict__ == other.__dict__

    def __ne__(self, other: 'RangeApplicationResp') -> bool:
        """Return `true` when self and other are not equal, false otherwise."""
        return not self == other

class RangeApplications():
    """
    range application.

    :attr bool success: Was the get successful.
    :attr List[List[str]] errors: errors.
    :attr List[List[str]] messages: messages.
    :attr List[RangeApplicationObject] result: Container for Range application
          objects.
    """

    def __init__(self,
                 success: bool,
                 errors: List[List[str]],
                 messages: List[List[str]],
                 result: List['RangeApplicationObject']) -> None:
        """
        Initialize a RangeApplications object.

        :param bool success: Was the get successful.
        :param List[List[str]] errors: errors.
        :param List[List[str]] messages: messages.
        :param List[RangeApplicationObject] result: Container for Range application
               objects.
        """
        self.success = success
        self.errors = errors
        self.messages = messages
        self.result = result

    @classmethod
    def from_dict(cls, _dict: Dict) -> 'RangeApplications':
        """Initialize a RangeApplications object from a json dictionary."""
        args = {}
        if 'success' in _dict:
            args['success'] = _dict.get('success')
        else:
            raise ValueError('Required property \'success\' not present in RangeApplications JSON')
        if 'errors' in _dict:
            args['errors'] = _dict.get('errors')
        else:
            raise ValueError('Required property \'errors\' not present in RangeApplications JSON')
        if 'messages' in _dict:
            args['messages'] = _dict.get('messages')
        else:
            raise ValueError('Required property \'messages\' not present in RangeApplications JSON')
        if 'result' in _dict:
            args['result'] = [RangeApplicationObject.from_dict(x) for x in _dict.get('result')]
        else:
            raise ValueError('Required property \'result\' not present in RangeApplications JSON')
        return cls(**args)

    @classmethod
    def _from_dict(cls, _dict):
        """Initialize a RangeApplications object from a json dictionary."""
        return cls.from_dict(_dict)

    def to_dict(self) -> Dict:
        """Return a json dictionary representing this model."""
        _dict = {}
        if hasattr(self, 'success') and self.success is not None:
            _dict['success'] = self.success
        if hasattr(self, 'errors') and self.errors is not None:
            _dict['errors'] = self.errors
        if hasattr(self, 'messages') and self.messages is not None:
            _dict['messages'] = self.messages
        if hasattr(self, 'result') and self.result is not None:
            _dict['result'] = [x.to_dict() for x in self.result]
        return _dict

    def _to_dict(self):
        """Return a json dictionary representing this model."""
        return self.to_dict()

    def __str__(self) -> str:
        """Return a `str` version of this RangeApplications object."""
        return json.dumps(self.to_dict(), indent=2)

    def __eq__(self, other: 'RangeApplications') -> bool:
        """Return `true` when self and other are equal, false otherwise."""
        if not isinstance(other, self.__class__):
            return False
        return self.__dict__ == other.__dict__

    def __ne__(self, other: 'RangeApplications') -> bool:
        """Return `true` when self and other are not equal, false otherwise."""
        return not self == other



# coding: utf-8

# (C) Copyright IBM Corp. 2020.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""
DNS records
"""

from enum import Enum
from typing import Dict, List
import json

from ibm_cloud_sdk_core import BaseService, DetailedResponse
from ibm_cloud_sdk_core.authenticators.authenticator import Authenticator
from ibm_cloud_sdk_core.get_authenticator import get_authenticator_from_environment

from .common import get_sdk_headers

##############################################################################
# Service
##############################################################################

class DnsRecordsV1(BaseService):
    """The DNS Records V1 service."""

    DEFAULT_SERVICE_URL = 'https://api.cis.cloud.ibm.com'
    DEFAULT_SERVICE_NAME = 'dns_records'

    @classmethod
    def new_instance(cls,
                     crn: str,
                     zone_identifier: str,
                     service_name: str = DEFAULT_SERVICE_NAME,
                    ) -> 'DnsRecordsV1':
        """
        Return a new client for the DNS Records service using the specified
               parameters and external configuration.

        :param str crn: Full crn of the service instance.

        :param str zone_identifier: Zone identifier (zone id).
        """
        if crn is None:
            raise ValueError('crn must be provided')
        if zone_identifier is None:
            raise ValueError('zone_identifier must be provided')

        authenticator = get_authenticator_from_environment(service_name)
        service = cls(
            crn,
            zone_identifier,
            authenticator
            )
        service.configure_service(service_name)
        return service

    def __init__(self,
                 crn: str,
                 zone_identifier: str,
                 authenticator: Authenticator = None,
                ) -> None:
        """
        Construct a new client for the DNS Records service.

        :param str crn: Full crn of the service instance.

        :param str zone_identifier: Zone identifier (zone id).

        :param Authenticator authenticator: The authenticator specifies the authentication mechanism.
               Get up to date information from https://github.com/IBM/python-sdk-core/blob/master/README.md
               about initializing the authenticator of your choice.
        """
        if crn is None:
            raise ValueError('crn must be provided')
        if zone_identifier is None:
            raise ValueError('zone_identifier must be provided')

        BaseService.__init__(self,
                             service_url=self.DEFAULT_SERVICE_URL,
                             authenticator=authenticator)
        self.crn = crn
        self.zone_identifier = zone_identifier


    #########################
    # DNS Records
    #########################


    def list_all_dns_records(self,
        *,
        type: str = None,
        name: str = None,
        content: str = None,
        page: int = None,
        per_page: int = None,
        order: str = None,
        direction: str = None,
        match: str = None,
        **kwargs
    ) -> DetailedResponse:
        """
        List all DNS records.

        List all DNS records for a given zone of a service instance.

        :param str type: (optional) Type of DNS records to display.
        :param str name: (optional) Value of name field to filter by.
        :param str content: (optional) Value of content field to filter by.
        :param int page: (optional) Page number of paginated results.
        :param int per_page: (optional) Maximum number of DNS records per page.
        :param str order: (optional) Field by which to order list of DNS records.
        :param str direction: (optional) Direction in which to order results
               [ascending/descending order].
        :param str match: (optional) Whether to match all (all) or atleast one
               search parameter (any).
        :param dict headers: A `dict` containing the request headers
        :return: A `DetailedResponse` containing the result, headers and HTTP status code.
        :rtype: DetailedResponse with `dict` result representing a `ListDnsrecordsResp` object
        """

        headers = {}
        sdk_headers = get_sdk_headers(service_name=self.DEFAULT_SERVICE_NAME,
                                      service_version='V1',
                                      operation_id='list_all_dns_records')
        headers.update(sdk_headers)

        params = {
            'type': type,
            'name': name,
            'content': content,
            'page': page,
            'per_page': per_page,
            'order': order,
            'direction': direction,
            'match': match
        }

        if 'headers' in kwargs:
            headers.update(kwargs.get('headers'))

        url = '/v1/{0}/zones/{1}/dns_records'.format(
            *self.encode_path_vars(self.crn, self.zone_identifier))
        request = self.prepare_request(method='GET',
                                       url=url,
                                       headers=headers,
                                       params=params)

        response = self.send(request)
        return response


    def create_dns_record(self,
        *,
        type: str = None,
        name: str = None,
        content: str = None,
        priority: int = None,
        data: object = None,
        **kwargs
    ) -> DetailedResponse:
        """
        Create DNS record.

        Add a new DNS record for a given zone for a given service instance.

        :param str type: (optional) dns record type.
        :param str name: (optional) Required for all record types except SRV.
        :param str content: (optional) dns record content.
        :param int priority: (optional) For MX records only.
        :param object data: (optional) For LOC, SRV and CAA records only.
        :param dict headers: A `dict` containing the request headers
        :return: A `DetailedResponse` containing the result, headers and HTTP status code.
        :rtype: DetailedResponse with `dict` result representing a `DnsrecordResp` object
        """

        headers = {}
        sdk_headers = get_sdk_headers(service_name=self.DEFAULT_SERVICE_NAME,
                                      service_version='V1',
                                      operation_id='create_dns_record')
        headers.update(sdk_headers)

        data = {
            'type': type,
            'name': name,
            'content': content,
            'priority': priority,
            'data': data
        }
        data = {k: v for (k, v) in data.items() if v is not None}
        data = json.dumps(data)
        headers['content-type'] = 'application/json'

        if 'headers' in kwargs:
            headers.update(kwargs.get('headers'))

        url = '/v1/{0}/zones/{1}/dns_records'.format(
            *self.encode_path_vars(self.crn, self.zone_identifier))
        request = self.prepare_request(method='POST',
                                       url=url,
                                       headers=headers,
                                       data=data)

        response = self.send(request)
        return response


    def delete_dns_record(self,
        dnsrecord_identifier: str,
        **kwargs
    ) -> DetailedResponse:
        """
        Delete DNS record.

        Delete a DNS record given its id.

        :param str dnsrecord_identifier: Identifier of DNS record.
        :param dict headers: A `dict` containing the request headers
        :return: A `DetailedResponse` containing the result, headers and HTTP status code.
        :rtype: DetailedResponse with `dict` result representing a `DeleteDnsrecordResp` object
        """

        if dnsrecord_identifier is None:
            raise ValueError('dnsrecord_identifier must be provided')
        headers = {}
        sdk_headers = get_sdk_headers(service_name=self.DEFAULT_SERVICE_NAME,
                                      service_version='V1',
                                      operation_id='delete_dns_record')
        headers.update(sdk_headers)

        if 'headers' in kwargs:
            headers.update(kwargs.get('headers'))

        url = '/v1/{0}/zones/{1}/dns_records/{2}'.format(
            *self.encode_path_vars(self.crn, self.zone_identifier, dnsrecord_identifier))
        request = self.prepare_request(method='DELETE',
                                       url=url,
                                       headers=headers)

        response = self.send(request)
        return response


    def get_dns_record(self,
        dnsrecord_identifier: str,
        **kwargs
    ) -> DetailedResponse:
        """
        Get DNS record.

        Get the details of a DNS record for a given zone under a given service instance.

        :param str dnsrecord_identifier: Identifier of DNS record.
        :param dict headers: A `dict` containing the request headers
        :return: A `DetailedResponse` containing the result, headers and HTTP status code.
        :rtype: DetailedResponse with `dict` result representing a `DnsrecordResp` object
        """

        if dnsrecord_identifier is None:
            raise ValueError('dnsrecord_identifier must be provided')
        headers = {}
        sdk_headers = get_sdk_headers(service_name=self.DEFAULT_SERVICE_NAME,
                                      service_version='V1',
                                      operation_id='get_dns_record')
        headers.update(sdk_headers)

        if 'headers' in kwargs:
            headers.update(kwargs.get('headers'))

        url = '/v1/{0}/zones/{1}/dns_records/{2}'.format(
            *self.encode_path_vars(self.crn, self.zone_identifier, dnsrecord_identifier))
        request = self.prepare_request(method='GET',
                                       url=url,
                                       headers=headers)

        response = self.send(request)
        return response


    def update_dns_record(self,
        dnsrecord_identifier: str,
        *,
        type: str = None,
        name: str = None,
        content: str = None,
        priority: int = None,
        proxied: bool = None,
        data: object = None,
        **kwargs
    ) -> DetailedResponse:
        """
        Update DNS record.

        Update an existing DNS record for a given zone under a given service instance.

        :param str dnsrecord_identifier: Identifier of DNS record.
        :param str type: (optional) dns record type.
        :param str name: (optional) Required for all record types except SRV.
        :param str content: (optional) content of dns record.
        :param int priority: (optional) For MX records only.
        :param bool proxied: (optional) proxied.
        :param object data: (optional) For LOC, SRV and CAA records only.
        :param dict headers: A `dict` containing the request headers
        :return: A `DetailedResponse` containing the result, headers and HTTP status code.
        :rtype: DetailedResponse with `dict` result representing a `DnsrecordResp` object
        """

        if dnsrecord_identifier is None:
            raise ValueError('dnsrecord_identifier must be provided')
        headers = {}
        sdk_headers = get_sdk_headers(service_name=self.DEFAULT_SERVICE_NAME,
                                      service_version='V1',
                                      operation_id='update_dns_record')
        headers.update(sdk_headers)

        data = {
            'type': type,
            'name': name,
            'content': content,
            'priority': priority,
            'proxied': proxied,
            'data': data
        }
        data = {k: v for (k, v) in data.items() if v is not None}
        data = json.dumps(data)
        headers['content-type'] = 'application/json'

        if 'headers' in kwargs:
            headers.update(kwargs.get('headers'))

        url = '/v1/{0}/zones/{1}/dns_records/{2}'.format(
            *self.encode_path_vars(self.crn, self.zone_identifier, dnsrecord_identifier))
        request = self.prepare_request(method='PUT',
                                       url=url,
                                       headers=headers,
                                       data=data)

        response = self.send(request)
        return response


class ListAllDnsRecordsEnums:
    """
    Enums for list_all_dns_records parameters.
    """

    class Order(str, Enum):
        """
        Field by which to order list of DNS records.
        """
        TYPE = 'type'
        NAME = 'name'
        CONTENT = 'content'
        TTL = 'ttl'
        PROXIED = 'proxied'
    class Direction(str, Enum):
        """
        Direction in which to order results [ascending/descending order].
        """
        ASC = 'asc'
        DESC = 'desc'
    class Match(str, Enum):
        """
        Whether to match all (all) or atleast one search parameter (any).
        """
        ANY = 'any'
        ALL = 'all'


##############################################################################
# Models
##############################################################################


class DeleteDnsrecordRespResult():
    """
    result.

    :attr str id: dns record id.
    """

    def __init__(self,
                 id: str) -> None:
        """
        Initialize a DeleteDnsrecordRespResult object.

        :param str id: dns record id.
        """
        self.id = id

    @classmethod
    def from_dict(cls, _dict: Dict) -> 'DeleteDnsrecordRespResult':
        """Initialize a DeleteDnsrecordRespResult object from a json dictionary."""
        args = {}
        if 'id' in _dict:
            args['id'] = _dict.get('id')
        else:
            raise ValueError('Required property \'id\' not present in DeleteDnsrecordRespResult JSON')
        return cls(**args)

    @classmethod
    def _from_dict(cls, _dict):
        """Initialize a DeleteDnsrecordRespResult object from a json dictionary."""
        return cls.from_dict(_dict)

    def to_dict(self) -> Dict:
        """Return a json dictionary representing this model."""
        _dict = {}
        if hasattr(self, 'id') and self.id is not None:
            _dict['id'] = self.id
        return _dict

    def _to_dict(self):
        """Return a json dictionary representing this model."""
        return self.to_dict()

    def __str__(self) -> str:
        """Return a `str` version of this DeleteDnsrecordRespResult object."""
        return json.dumps(self.to_dict(), indent=2)

    def __eq__(self, other: 'DeleteDnsrecordRespResult') -> bool:
        """Return `true` when self and other are equal, false otherwise."""
        if not isinstance(other, self.__class__):
            return False
        return self.__dict__ == other.__dict__

    def __ne__(self, other: 'DeleteDnsrecordRespResult') -> bool:
        """Return `true` when self and other are not equal, false otherwise."""
        return not self == other

class DeleteDnsrecordResp():
    """
    dns record delete response.

    :attr bool success: success response.
    :attr List[List[str]] errors: errors.
    :attr List[List[str]] messages: messages.
    :attr DeleteDnsrecordRespResult result: result.
    """

    def __init__(self,
                 success: bool,
                 errors: List[List[str]],
                 messages: List[List[str]],
                 result: 'DeleteDnsrecordRespResult') -> None:
        """
        Initialize a DeleteDnsrecordResp object.

        :param bool success: success response.
        :param List[List[str]] errors: errors.
        :param List[List[str]] messages: messages.
        :param DeleteDnsrecordRespResult result: result.
        """
        self.success = success
        self.errors = errors
        self.messages = messages
        self.result = result

    @classmethod
    def from_dict(cls, _dict: Dict) -> 'DeleteDnsrecordResp':
        """Initialize a DeleteDnsrecordResp object from a json dictionary."""
        args = {}
        if 'success' in _dict:
            args['success'] = _dict.get('success')
        else:
            raise ValueError('Required property \'success\' not present in DeleteDnsrecordResp JSON')
        if 'errors' in _dict:
            args['errors'] = _dict.get('errors')
        else:
            raise ValueError('Required property \'errors\' not present in DeleteDnsrecordResp JSON')
        if 'messages' in _dict:
            args['messages'] = _dict.get('messages')
        else:
            raise ValueError('Required property \'messages\' not present in DeleteDnsrecordResp JSON')
        if 'result' in _dict:
            args['result'] = DeleteDnsrecordRespResult.from_dict(_dict.get('result'))
        else:
            raise ValueError('Required property \'result\' not present in DeleteDnsrecordResp JSON')
        return cls(**args)

    @classmethod
    def _from_dict(cls, _dict):
        """Initialize a DeleteDnsrecordResp object from a json dictionary."""
        return cls.from_dict(_dict)

    def to_dict(self) -> Dict:
        """Return a json dictionary representing this model."""
        _dict = {}
        if hasattr(self, 'success') and self.success is not None:
            _dict['success'] = self.success
        if hasattr(self, 'errors') and self.errors is not None:
            _dict['errors'] = self.errors
        if hasattr(self, 'messages') and self.messages is not None:
            _dict['messages'] = self.messages
        if hasattr(self, 'result') and self.result is not None:
            _dict['result'] = self.result.to_dict()
        return _dict

    def _to_dict(self):
        """Return a json dictionary representing this model."""
        return self.to_dict()

    def __str__(self) -> str:
        """Return a `str` version of this DeleteDnsrecordResp object."""
        return json.dumps(self.to_dict(), indent=2)

    def __eq__(self, other: 'DeleteDnsrecordResp') -> bool:
        """Return `true` when self and other are equal, false otherwise."""
        if not isinstance(other, self.__class__):
            return False
        return self.__dict__ == other.__dict__

    def __ne__(self, other: 'DeleteDnsrecordResp') -> bool:
        """Return `true` when self and other are not equal, false otherwise."""
        return not self == other

class DnsrecordDetails():
    """
    dns record details.

    :attr str id: (optional) dns record identifier.
    :attr str created_on: (optional) created on.
    :attr str modified_on: (optional) modified date.
    :attr str name: (optional) dns record name.
    :attr str type: (optional) dns record type.
    :attr str content: (optional) dns record content.
    :attr str zone_id: (optional) zone identifier.
    :attr str zone_name: (optional) zone name.
    :attr bool proxiable: (optional) proxiable.
    :attr bool proxied: (optional) proxied.
    :attr int ttl: (optional) dns record ttl value.
    :attr int priority: (optional) Relevant only to MX type records.
    :attr object data: (optional) Data details for the DNS record. Only for LOC,
          SRV, CAA records.
    """

    def __init__(self,
                 *,
                 id: str = None,
                 created_on: str = None,
                 modified_on: str = None,
                 name: str = None,
                 type: str = None,
                 content: str = None,
                 zone_id: str = None,
                 zone_name: str = None,
                 proxiable: bool = None,
                 proxied: bool = None,
                 ttl: int = None,
                 priority: int = None,
                 data: object = None) -> None:
        """
        Initialize a DnsrecordDetails object.

        :param str id: (optional) dns record identifier.
        :param str created_on: (optional) created on.
        :param str modified_on: (optional) modified date.
        :param str name: (optional) dns record name.
        :param str type: (optional) dns record type.
        :param str content: (optional) dns record content.
        :param str zone_id: (optional) zone identifier.
        :param str zone_name: (optional) zone name.
        :param bool proxiable: (optional) proxiable.
        :param bool proxied: (optional) proxied.
        :param int ttl: (optional) dns record ttl value.
        :param int priority: (optional) Relevant only to MX type records.
        :param object data: (optional) Data details for the DNS record. Only for
               LOC, SRV, CAA records.
        """
        self.id = id
        self.created_on = created_on
        self.modified_on = modified_on
        self.name = name
        self.type = type
        self.content = content
        self.zone_id = zone_id
        self.zone_name = zone_name
        self.proxiable = proxiable
        self.proxied = proxied
        self.ttl = ttl
        self.priority = priority
        self.data = data

    @classmethod
    def from_dict(cls, _dict: Dict) -> 'DnsrecordDetails':
        """Initialize a DnsrecordDetails object from a json dictionary."""
        args = {}
        if 'id' in _dict:
            args['id'] = _dict.get('id')
        if 'created_on' in _dict:
            args['created_on'] = _dict.get('created_on')
        if 'modified_on' in _dict:
            args['modified_on'] = _dict.get('modified_on')
        if 'name' in _dict:
            args['name'] = _dict.get('name')
        if 'type' in _dict:
            args['type'] = _dict.get('type')
        if 'content' in _dict:
            args['content'] = _dict.get('content')
        if 'zone_id' in _dict:
            args['zone_id'] = _dict.get('zone_id')
        if 'zone_name' in _dict:
            args['zone_name'] = _dict.get('zone_name')
        if 'proxiable' in _dict:
            args['proxiable'] = _dict.get('proxiable')
        if 'proxied' in _dict:
            args['proxied'] = _dict.get('proxied')
        if 'ttl' in _dict:
            args['ttl'] = _dict.get('ttl')
        if 'priority' in _dict:
            args['priority'] = _dict.get('priority')
        if 'data' in _dict:
            args['data'] = _dict.get('data')
        return cls(**args)

    @classmethod
    def _from_dict(cls, _dict):
        """Initialize a DnsrecordDetails object from a json dictionary."""
        return cls.from_dict(_dict)

    def to_dict(self) -> Dict:
        """Return a json dictionary representing this model."""
        _dict = {}
        if hasattr(self, 'id') and self.id is not None:
            _dict['id'] = self.id
        if hasattr(self, 'created_on') and self.created_on is not None:
            _dict['created_on'] = self.created_on
        if hasattr(self, 'modified_on') and self.modified_on is not None:
            _dict['modified_on'] = self.modified_on
        if hasattr(self, 'name') and self.name is not None:
            _dict['name'] = self.name
        if hasattr(self, 'type') and self.type is not None:
            _dict['type'] = self.type
        if hasattr(self, 'content') and self.content is not None:
            _dict['content'] = self.content
        if hasattr(self, 'zone_id') and self.zone_id is not None:
            _dict['zone_id'] = self.zone_id
        if hasattr(self, 'zone_name') and self.zone_name is not None:
            _dict['zone_name'] = self.zone_name
        if hasattr(self, 'proxiable') and self.proxiable is not None:
            _dict['proxiable'] = self.proxiable
        if hasattr(self, 'proxied') and self.proxied is not None:
            _dict['proxied'] = self.proxied
        if hasattr(self, 'ttl') and self.ttl is not None:
            _dict['ttl'] = self.ttl
        if hasattr(self, 'priority') and self.priority is not None:
            _dict['priority'] = self.priority
        if hasattr(self, 'data') and self.data is not None:
            _dict['data'] = self.data
        return _dict

    def _to_dict(self):
        """Return a json dictionary representing this model."""
        return self.to_dict()

    def __str__(self) -> str:
        """Return a `str` version of this DnsrecordDetails object."""
        return json.dumps(self.to_dict(), indent=2)

    def __eq__(self, other: 'DnsrecordDetails') -> bool:
        """Return `true` when self and other are equal, false otherwise."""
        if not isinstance(other, self.__class__):
            return False
        return self.__dict__ == other.__dict__

    def __ne__(self, other: 'DnsrecordDetails') -> bool:
        """Return `true` when self and other are not equal, false otherwise."""
        return not self == other

    class TypeEnum(str, Enum):
        """
        dns record type.
        """
        A = 'A'
        AAAA = 'AAAA'
        CNAME = 'CNAME'
        NS = 'NS'
        MX = 'MX'
        TXT = 'TXT'
        LOC = 'LOC'
        SRV = 'SRV'
        SPF = 'SPF'
        CAA = 'CAA'


class DnsrecordResp():
    """
    dns record response.

    :attr bool success: success response.
    :attr List[List[str]] errors: errors.
    :attr List[List[str]] messages: messages.
    :attr DnsrecordDetails result: dns record details.
    """

    def __init__(self,
                 success: bool,
                 errors: List[List[str]],
                 messages: List[List[str]],
                 result: 'DnsrecordDetails') -> None:
        """
        Initialize a DnsrecordResp object.

        :param bool success: success response.
        :param List[List[str]] errors: errors.
        :param List[List[str]] messages: messages.
        :param DnsrecordDetails result: dns record details.
        """
        self.success = success
        self.errors = errors
        self.messages = messages
        self.result = result

    @classmethod
    def from_dict(cls, _dict: Dict) -> 'DnsrecordResp':
        """Initialize a DnsrecordResp object from a json dictionary."""
        args = {}
        if 'success' in _dict:
            args['success'] = _dict.get('success')
        else:
            raise ValueError('Required property \'success\' not present in DnsrecordResp JSON')
        if 'errors' in _dict:
            args['errors'] = _dict.get('errors')
        else:
            raise ValueError('Required property \'errors\' not present in DnsrecordResp JSON')
        if 'messages' in _dict:
            args['messages'] = _dict.get('messages')
        else:
            raise ValueError('Required property \'messages\' not present in DnsrecordResp JSON')
        if 'result' in _dict:
            args['result'] = DnsrecordDetails.from_dict(_dict.get('result'))
        else:
            raise ValueError('Required property \'result\' not present in DnsrecordResp JSON')
        return cls(**args)

    @classmethod
    def _from_dict(cls, _dict):
        """Initialize a DnsrecordResp object from a json dictionary."""
        return cls.from_dict(_dict)

    def to_dict(self) -> Dict:
        """Return a json dictionary representing this model."""
        _dict = {}
        if hasattr(self, 'success') and self.success is not None:
            _dict['success'] = self.success
        if hasattr(self, 'errors') and self.errors is not None:
            _dict['errors'] = self.errors
        if hasattr(self, 'messages') and self.messages is not None:
            _dict['messages'] = self.messages
        if hasattr(self, 'result') and self.result is not None:
            _dict['result'] = self.result.to_dict()
        return _dict

    def _to_dict(self):
        """Return a json dictionary representing this model."""
        return self.to_dict()

    def __str__(self) -> str:
        """Return a `str` version of this DnsrecordResp object."""
        return json.dumps(self.to_dict(), indent=2)

    def __eq__(self, other: 'DnsrecordResp') -> bool:
        """Return `true` when self and other are equal, false otherwise."""
        if not isinstance(other, self.__class__):
            return False
        return self.__dict__ == other.__dict__

    def __ne__(self, other: 'DnsrecordResp') -> bool:
        """Return `true` when self and other are not equal, false otherwise."""
        return not self == other

class ListDnsrecordsResp():
    """
    dns records list response.

    :attr bool success: success response.
    :attr List[List[str]] errors: errors.
    :attr List[List[str]] messages: messages.
    :attr List[DnsrecordDetails] result: dns record list.
    :attr ResultInfo result_info: result information.
    """

    def __init__(self,
                 success: bool,
                 errors: List[List[str]],
                 messages: List[List[str]],
                 result: List['DnsrecordDetails'],
                 result_info: 'ResultInfo') -> None:
        """
        Initialize a ListDnsrecordsResp object.

        :param bool success: success response.
        :param List[List[str]] errors: errors.
        :param List[List[str]] messages: messages.
        :param List[DnsrecordDetails] result: dns record list.
        :param ResultInfo result_info: result information.
        """
        self.success = success
        self.errors = errors
        self.messages = messages
        self.result = result
        self.result_info = result_info

    @classmethod
    def from_dict(cls, _dict: Dict) -> 'ListDnsrecordsResp':
        """Initialize a ListDnsrecordsResp object from a json dictionary."""
        args = {}
        if 'success' in _dict:
            args['success'] = _dict.get('success')
        else:
            raise ValueError('Required property \'success\' not present in ListDnsrecordsResp JSON')
        if 'errors' in _dict:
            args['errors'] = _dict.get('errors')
        else:
            raise ValueError('Required property \'errors\' not present in ListDnsrecordsResp JSON')
        if 'messages' in _dict:
            args['messages'] = _dict.get('messages')
        else:
            raise ValueError('Required property \'messages\' not present in ListDnsrecordsResp JSON')
        if 'result' in _dict:
            args['result'] = [DnsrecordDetails.from_dict(x) for x in _dict.get('result')]
        else:
            raise ValueError('Required property \'result\' not present in ListDnsrecordsResp JSON')
        if 'result_info' in _dict:
            args['result_info'] = ResultInfo.from_dict(_dict.get('result_info'))
        else:
            raise ValueError('Required property \'result_info\' not present in ListDnsrecordsResp JSON')
        return cls(**args)

    @classmethod
    def _from_dict(cls, _dict):
        """Initialize a ListDnsrecordsResp object from a json dictionary."""
        return cls.from_dict(_dict)

    def to_dict(self) -> Dict:
        """Return a json dictionary representing this model."""
        _dict = {}
        if hasattr(self, 'success') and self.success is not None:
            _dict['success'] = self.success
        if hasattr(self, 'errors') and self.errors is not None:
            _dict['errors'] = self.errors
        if hasattr(self, 'messages') and self.messages is not None:
            _dict['messages'] = self.messages
        if hasattr(self, 'result') and self.result is not None:
            _dict['result'] = [x.to_dict() for x in self.result]
        if hasattr(self, 'result_info') and self.result_info is not None:
            _dict['result_info'] = self.result_info.to_dict()
        return _dict

    def _to_dict(self):
        """Return a json dictionary representing this model."""
        return self.to_dict()

    def __str__(self) -> str:
        """Return a `str` version of this ListDnsrecordsResp object."""
        return json.dumps(self.to_dict(), indent=2)

    def __eq__(self, other: 'ListDnsrecordsResp') -> bool:
        """Return `true` when self and other are equal, false otherwise."""
        if not isinstance(other, self.__class__):
            return False
        return self.__dict__ == other.__dict__

    def __ne__(self, other: 'ListDnsrecordsResp') -> bool:
        """Return `true` when self and other are not equal, false otherwise."""
        return not self == other

class ResultInfo():
    """
    result information.

    :attr int page: page.
    :attr int per_page: per page.
    :attr int count: count.
    :attr int total_count: total count.
    """

    def __init__(self,
                 page: int,
                 per_page: int,
                 count: int,
                 total_count: int) -> None:
        """
        Initialize a ResultInfo object.

        :param int page: page.
        :param int per_page: per page.
        :param int count: count.
        :param int total_count: total count.
        """
        self.page = page
        self.per_page = per_page
        self.count = count
        self.total_count = total_count

    @classmethod
    def from_dict(cls, _dict: Dict) -> 'ResultInfo':
        """Initialize a ResultInfo object from a json dictionary."""
        args = {}
        if 'page' in _dict:
            args['page'] = _dict.get('page')
        else:
            raise ValueError('Required property \'page\' not present in ResultInfo JSON')
        if 'per_page' in _dict:
            args['per_page'] = _dict.get('per_page')
        else:
            raise ValueError('Required property \'per_page\' not present in ResultInfo JSON')
        if 'count' in _dict:
            args['count'] = _dict.get('count')
        else:
            raise ValueError('Required property \'count\' not present in ResultInfo JSON')
        if 'total_count' in _dict:
            args['total_count'] = _dict.get('total_count')
        else:
            raise ValueError('Required property \'total_count\' not present in ResultInfo JSON')
        return cls(**args)

    @classmethod
    def _from_dict(cls, _dict):
        """Initialize a ResultInfo object from a json dictionary."""
        return cls.from_dict(_dict)

    def to_dict(self) -> Dict:
        """Return a json dictionary representing this model."""
        _dict = {}
        if hasattr(self, 'page') and self.page is not None:
            _dict['page'] = self.page
        if hasattr(self, 'per_page') and self.per_page is not None:
            _dict['per_page'] = self.per_page
        if hasattr(self, 'count') and self.count is not None:
            _dict['count'] = self.count
        if hasattr(self, 'total_count') and self.total_count is not None:
            _dict['total_count'] = self.total_count
        return _dict

    def _to_dict(self):
        """Return a json dictionary representing this model."""
        return self.to_dict()

    def __str__(self) -> str:
        """Return a `str` version of this ResultInfo object."""
        return json.dumps(self.to_dict(), indent=2)

    def __eq__(self, other: 'ResultInfo') -> bool:
        """Return `true` when self and other are equal, false otherwise."""
        if not isinstance(other, self.__class__):
            return False
        return self.__dict__ == other.__dict__

    def __ne__(self, other: 'ResultInfo') -> bool:
        """Return `true` when self and other are not equal, false otherwise."""
        return not self == other



# coding: utf-8

# (C) Copyright IBM Corp. 2020.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""
This document describes CIS WAF Rule Groups API.
"""

from enum import Enum
from typing import Dict, List
import json

from ibm_cloud_sdk_core import BaseService, DetailedResponse
from ibm_cloud_sdk_core.authenticators.authenticator import Authenticator
from ibm_cloud_sdk_core.get_authenticator import get_authenticator_from_environment

from .common import get_sdk_headers

##############################################################################
# Service
##############################################################################

class WafRuleGroupsApiV1(BaseService):
    """The WAF Rule Groups API V1 service."""

    DEFAULT_SERVICE_URL = 'https://api.cis.cloud.ibm.com'
    DEFAULT_SERVICE_NAME = 'waf_rule_groups_api'

    @classmethod
    def new_instance(cls,
                     crn: str,
                     zone_id: str,
                     service_name: str = DEFAULT_SERVICE_NAME,
                    ) -> 'WafRuleGroupsApiV1':
        """
        Return a new client for the WAF Rule Groups API service using the specified
               parameters and external configuration.

        :param str crn: cloud resource name.

        :param str zone_id: Zone ID.
        """
        if crn is None:
            raise ValueError('crn must be provided')
        if zone_id is None:
            raise ValueError('zone_id must be provided')

        authenticator = get_authenticator_from_environment(service_name)
        service = cls(
            crn,
            zone_id,
            authenticator
            )
        service.configure_service(service_name)
        return service

    def __init__(self,
                 crn: str,
                 zone_id: str,
                 authenticator: Authenticator = None,
                ) -> None:
        """
        Construct a new client for the WAF Rule Groups API service.

        :param str crn: cloud resource name.

        :param str zone_id: Zone ID.

        :param Authenticator authenticator: The authenticator specifies the authentication mechanism.
               Get up to date information from https://github.com/IBM/python-sdk-core/blob/master/README.md
               about initializing the authenticator of your choice.
        """
        if crn is None:
            raise ValueError('crn must be provided')
        if zone_id is None:
            raise ValueError('zone_id must be provided')

        BaseService.__init__(self,
                             service_url=self.DEFAULT_SERVICE_URL,
                             authenticator=authenticator)
        self.crn = crn
        self.zone_id = zone_id


    #########################
    # WAF Rule Groups
    #########################


    def list_waf_rule_groups(self,
        pkg_id: str,
        *,
        name: str = None,
        mode: str = None,
        rules_count: str = None,
        page: int = None,
        per_page: int = None,
        order: str = None,
        direction: str = None,
        match: str = None,
        **kwargs
    ) -> DetailedResponse:
        """
        List all WAF rule groups.

        List all WAF rule groups contained within a package.

        :param str pkg_id: Package ID.
        :param str name: (optional) Name of the firewall package.
        :param str mode: (optional) Whether or not the rules contained within this
               group are configurable/usable.
        :param str rules_count: (optional) How many rules are contained within this
               group.
        :param int page: (optional) Page number of paginated results.
        :param int per_page: (optional) Number of packages per page.
        :param str order: (optional) Field to order packages by.
        :param str direction: (optional) Direction to order packages.
        :param str match: (optional) Whether to match all search requirements or at
               least one (any).
        :param dict headers: A `dict` containing the request headers
        :return: A `DetailedResponse` containing the result, headers and HTTP status code.
        :rtype: DetailedResponse with `dict` result representing a `WafGroupsResponse` object
        """

        if pkg_id is None:
            raise ValueError('pkg_id must be provided')
        headers = {}
        sdk_headers = get_sdk_headers(service_name=self.DEFAULT_SERVICE_NAME,
                                      service_version='V1',
                                      operation_id='list_waf_rule_groups')
        headers.update(sdk_headers)

        params = {
            'name': name,
            'mode': mode,
            'rules_count': rules_count,
            'page': page,
            'per_page': per_page,
            'order': order,
            'direction': direction,
            'match': match
        }

        if 'headers' in kwargs:
            headers.update(kwargs.get('headers'))

        url = '/v1/{0}/zones/{1}/firewall/waf/packages/{2}/groups'.format(
            *self.encode_path_vars(self.crn, self.zone_id, pkg_id))
        request = self.prepare_request(method='GET',
                                       url=url,
                                       headers=headers,
                                       params=params)

        response = self.send(request)
        return response


    def get_waf_rule_group(self,
        pkg_id: str,
        group_id: str,
        **kwargs
    ) -> DetailedResponse:
        """
        Get WAF rule group.

        Get a single WAF rule group.

        :param str pkg_id: Package ID.
        :param str group_id: Group ID.
        :param dict headers: A `dict` containing the request headers
        :return: A `DetailedResponse` containing the result, headers and HTTP status code.
        :rtype: DetailedResponse with `dict` result representing a `WafGroupResponse` object
        """

        if pkg_id is None:
            raise ValueError('pkg_id must be provided')
        if group_id is None:
            raise ValueError('group_id must be provided')
        headers = {}
        sdk_headers = get_sdk_headers(service_name=self.DEFAULT_SERVICE_NAME,
                                      service_version='V1',
                                      operation_id='get_waf_rule_group')
        headers.update(sdk_headers)

        if 'headers' in kwargs:
            headers.update(kwargs.get('headers'))

        url = '/v1/{0}/zones/{1}/firewall/waf/packages/{2}/groups/{3}'.format(
            *self.encode_path_vars(self.crn, self.zone_id, pkg_id, group_id))
        request = self.prepare_request(method='GET',
                                       url=url,
                                       headers=headers)

        response = self.send(request)
        return response


    def update_waf_rule_group(self,
        pkg_id: str,
        group_id: str,
        *,
        mode: str = None,
        **kwargs
    ) -> DetailedResponse:
        """
        Update WAF rule group.

        Update the state of a WAF rule group.

        :param str pkg_id: Package ID.
        :param str group_id: Group ID.
        :param str mode: (optional) Whether or not the rules contained within this
               group are configurable/usable.
        :param dict headers: A `dict` containing the request headers
        :return: A `DetailedResponse` containing the result, headers and HTTP status code.
        :rtype: DetailedResponse with `dict` result representing a `WafGroupResponse` object
        """

        if pkg_id is None:
            raise ValueError('pkg_id must be provided')
        if group_id is None:
            raise ValueError('group_id must be provided')
        headers = {}
        sdk_headers = get_sdk_headers(service_name=self.DEFAULT_SERVICE_NAME,
                                      service_version='V1',
                                      operation_id='update_waf_rule_group')
        headers.update(sdk_headers)

        data = {
            'mode': mode
        }
        data = {k: v for (k, v) in data.items() if v is not None}
        data = json.dumps(data)
        headers['content-type'] = 'application/json'

        if 'headers' in kwargs:
            headers.update(kwargs.get('headers'))

        url = '/v1/{0}/zones/{1}/firewall/waf/packages/{2}/groups/{3}'.format(
            *self.encode_path_vars(self.crn, self.zone_id, pkg_id, group_id))
        request = self.prepare_request(method='PATCH',
                                       url=url,
                                       headers=headers,
                                       data=data)

        response = self.send(request)
        return response


class ListWafRuleGroupsEnums:
    """
    Enums for list_waf_rule_groups parameters.
    """

    class Mode(str, Enum):
        """
        Whether or not the rules contained within this group are configurable/usable.
        """
        ON = 'on'
        OFF = 'off'
    class Direction(str, Enum):
        """
        Direction to order packages.
        """
        DESC = 'desc'
        ASC = 'asc'
    class Match(str, Enum):
        """
        Whether to match all search requirements or at least one (any).
        """
        ALL = 'all'
        ANY = 'any'


##############################################################################
# Models
##############################################################################


class WafGroupResponseResultInfo():
    """
    Statistics of results.

    :attr int page: Page number.
    :attr int per_page: Number of results per page.
    :attr int count: Number of results.
    :attr int total_count: Total number of results.
    """

    def __init__(self,
                 page: int,
                 per_page: int,
                 count: int,
                 total_count: int) -> None:
        """
        Initialize a WafGroupResponseResultInfo object.

        :param int page: Page number.
        :param int per_page: Number of results per page.
        :param int count: Number of results.
        :param int total_count: Total number of results.
        """
        self.page = page
        self.per_page = per_page
        self.count = count
        self.total_count = total_count

    @classmethod
    def from_dict(cls, _dict: Dict) -> 'WafGroupResponseResultInfo':
        """Initialize a WafGroupResponseResultInfo object from a json dictionary."""
        args = {}
        if 'page' in _dict:
            args['page'] = _dict.get('page')
        else:
            raise ValueError('Required property \'page\' not present in WafGroupResponseResultInfo JSON')
        if 'per_page' in _dict:
            args['per_page'] = _dict.get('per_page')
        else:
            raise ValueError('Required property \'per_page\' not present in WafGroupResponseResultInfo JSON')
        if 'count' in _dict:
            args['count'] = _dict.get('count')
        else:
            raise ValueError('Required property \'count\' not present in WafGroupResponseResultInfo JSON')
        if 'total_count' in _dict:
            args['total_count'] = _dict.get('total_count')
        else:
            raise ValueError('Required property \'total_count\' not present in WafGroupResponseResultInfo JSON')
        return cls(**args)

    @classmethod
    def _from_dict(cls, _dict):
        """Initialize a WafGroupResponseResultInfo object from a json dictionary."""
        return cls.from_dict(_dict)

    def to_dict(self) -> Dict:
        """Return a json dictionary representing this model."""
        _dict = {}
        if hasattr(self, 'page') and self.page is not None:
            _dict['page'] = self.page
        if hasattr(self, 'per_page') and self.per_page is not None:
            _dict['per_page'] = self.per_page
        if hasattr(self, 'count') and self.count is not None:
            _dict['count'] = self.count
        if hasattr(self, 'total_count') and self.total_count is not None:
            _dict['total_count'] = self.total_count
        return _dict

    def _to_dict(self):
        """Return a json dictionary representing this model."""
        return self.to_dict()

    def __str__(self) -> str:
        """Return a `str` version of this WafGroupResponseResultInfo object."""
        return json.dumps(self.to_dict(), indent=2)

    def __eq__(self, other: 'WafGroupResponseResultInfo') -> bool:
        """Return `true` when self and other are equal, false otherwise."""
        if not isinstance(other, self.__class__):
            return False
        return self.__dict__ == other.__dict__

    def __ne__(self, other: 'WafGroupResponseResultInfo') -> bool:
        """Return `true` when self and other are not equal, false otherwise."""
        return not self == other

class WafGroupsResponseResultInfo():
    """
    Statistics of results.

    :attr int page: Page number.
    :attr int per_page: Number of results per page.
    :attr int count: Number of results.
    :attr int total_count: Total number of results.
    """

    def __init__(self,
                 page: int,
                 per_page: int,
                 count: int,
                 total_count: int) -> None:
        """
        Initialize a WafGroupsResponseResultInfo object.

        :param int page: Page number.
        :param int per_page: Number of results per page.
        :param int count: Number of results.
        :param int total_count: Total number of results.
        """
        self.page = page
        self.per_page = per_page
        self.count = count
        self.total_count = total_count

    @classmethod
    def from_dict(cls, _dict: Dict) -> 'WafGroupsResponseResultInfo':
        """Initialize a WafGroupsResponseResultInfo object from a json dictionary."""
        args = {}
        if 'page' in _dict:
            args['page'] = _dict.get('page')
        else:
            raise ValueError('Required property \'page\' not present in WafGroupsResponseResultInfo JSON')
        if 'per_page' in _dict:
            args['per_page'] = _dict.get('per_page')
        else:
            raise ValueError('Required property \'per_page\' not present in WafGroupsResponseResultInfo JSON')
        if 'count' in _dict:
            args['count'] = _dict.get('count')
        else:
            raise ValueError('Required property \'count\' not present in WafGroupsResponseResultInfo JSON')
        if 'total_count' in _dict:
            args['total_count'] = _dict.get('total_count')
        else:
            raise ValueError('Required property \'total_count\' not present in WafGroupsResponseResultInfo JSON')
        return cls(**args)

    @classmethod
    def _from_dict(cls, _dict):
        """Initialize a WafGroupsResponseResultInfo object from a json dictionary."""
        return cls.from_dict(_dict)

    def to_dict(self) -> Dict:
        """Return a json dictionary representing this model."""
        _dict = {}
        if hasattr(self, 'page') and self.page is not None:
            _dict['page'] = self.page
        if hasattr(self, 'per_page') and self.per_page is not None:
            _dict['per_page'] = self.per_page
        if hasattr(self, 'count') and self.count is not None:
            _dict['count'] = self.count
        if hasattr(self, 'total_count') and self.total_count is not None:
            _dict['total_count'] = self.total_count
        return _dict

    def _to_dict(self):
        """Return a json dictionary representing this model."""
        return self.to_dict()

    def __str__(self) -> str:
        """Return a `str` version of this WafGroupsResponseResultInfo object."""
        return json.dumps(self.to_dict(), indent=2)

    def __eq__(self, other: 'WafGroupsResponseResultInfo') -> bool:
        """Return `true` when self and other are equal, false otherwise."""
        if not isinstance(other, self.__class__):
            return False
        return self.__dict__ == other.__dict__

    def __ne__(self, other: 'WafGroupsResponseResultInfo') -> bool:
        """Return `true` when self and other are not equal, false otherwise."""
        return not self == other

class WafGroupResponse():
    """
    waf group response.

    :attr bool success: Was operation successful.
    :attr List[List[str]] errors: Array of errors encountered.
    :attr List[List[str]] messages: Array of messages returned.
    :attr WafRuleProperties result: waf rule properties.
    :attr WafGroupResponseResultInfo result_info: Statistics of results.
    """

    def __init__(self,
                 success: bool,
                 errors: List[List[str]],
                 messages: List[List[str]],
                 result: 'WafRuleProperties',
                 result_info: 'WafGroupResponseResultInfo') -> None:
        """
        Initialize a WafGroupResponse object.

        :param bool success: Was operation successful.
        :param List[List[str]] errors: Array of errors encountered.
        :param List[List[str]] messages: Array of messages returned.
        :param WafRuleProperties result: waf rule properties.
        :param WafGroupResponseResultInfo result_info: Statistics of results.
        """
        self.success = success
        self.errors = errors
        self.messages = messages
        self.result = result
        self.result_info = result_info

    @classmethod
    def from_dict(cls, _dict: Dict) -> 'WafGroupResponse':
        """Initialize a WafGroupResponse object from a json dictionary."""
        args = {}
        if 'success' in _dict:
            args['success'] = _dict.get('success')
        else:
            raise ValueError('Required property \'success\' not present in WafGroupResponse JSON')
        if 'errors' in _dict:
            args['errors'] = _dict.get('errors')
        else:
            raise ValueError('Required property \'errors\' not present in WafGroupResponse JSON')
        if 'messages' in _dict:
            args['messages'] = _dict.get('messages')
        else:
            raise ValueError('Required property \'messages\' not present in WafGroupResponse JSON')
        if 'result' in _dict:
            args['result'] = WafRuleProperties.from_dict(_dict.get('result'))
        else:
            raise ValueError('Required property \'result\' not present in WafGroupResponse JSON')
        if 'result_info' in _dict:
            args['result_info'] = WafGroupResponseResultInfo.from_dict(_dict.get('result_info'))
        else:
            raise ValueError('Required property \'result_info\' not present in WafGroupResponse JSON')
        return cls(**args)

    @classmethod
    def _from_dict(cls, _dict):
        """Initialize a WafGroupResponse object from a json dictionary."""
        return cls.from_dict(_dict)

    def to_dict(self) -> Dict:
        """Return a json dictionary representing this model."""
        _dict = {}
        if hasattr(self, 'success') and self.success is not None:
            _dict['success'] = self.success
        if hasattr(self, 'errors') and self.errors is not None:
            _dict['errors'] = self.errors
        if hasattr(self, 'messages') and self.messages is not None:
            _dict['messages'] = self.messages
        if hasattr(self, 'result') and self.result is not None:
            _dict['result'] = self.result.to_dict()
        if hasattr(self, 'result_info') and self.result_info is not None:
            _dict['result_info'] = self.result_info.to_dict()
        return _dict

    def _to_dict(self):
        """Return a json dictionary representing this model."""
        return self.to_dict()

    def __str__(self) -> str:
        """Return a `str` version of this WafGroupResponse object."""
        return json.dumps(self.to_dict(), indent=2)

    def __eq__(self, other: 'WafGroupResponse') -> bool:
        """Return `true` when self and other are equal, false otherwise."""
        if not isinstance(other, self.__class__):
            return False
        return self.__dict__ == other.__dict__

    def __ne__(self, other: 'WafGroupResponse') -> bool:
        """Return `true` when self and other are not equal, false otherwise."""
        return not self == other

class WafGroupsResponse():
    """
    waf groups response.

    :attr bool success: Was operation successful.
    :attr List[List[str]] errors: Array of errors encountered.
    :attr List[List[str]] messages: Array of messages returned.
    :attr List[WafRuleProperties] result: Container for response information.
    :attr WafGroupsResponseResultInfo result_info: Statistics of results.
    """

    def __init__(self,
                 success: bool,
                 errors: List[List[str]],
                 messages: List[List[str]],
                 result: List['WafRuleProperties'],
                 result_info: 'WafGroupsResponseResultInfo') -> None:
        """
        Initialize a WafGroupsResponse object.

        :param bool success: Was operation successful.
        :param List[List[str]] errors: Array of errors encountered.
        :param List[List[str]] messages: Array of messages returned.
        :param List[WafRuleProperties] result: Container for response information.
        :param WafGroupsResponseResultInfo result_info: Statistics of results.
        """
        self.success = success
        self.errors = errors
        self.messages = messages
        self.result = result
        self.result_info = result_info

    @classmethod
    def from_dict(cls, _dict: Dict) -> 'WafGroupsResponse':
        """Initialize a WafGroupsResponse object from a json dictionary."""
        args = {}
        if 'success' in _dict:
            args['success'] = _dict.get('success')
        else:
            raise ValueError('Required property \'success\' not present in WafGroupsResponse JSON')
        if 'errors' in _dict:
            args['errors'] = _dict.get('errors')
        else:
            raise ValueError('Required property \'errors\' not present in WafGroupsResponse JSON')
        if 'messages' in _dict:
            args['messages'] = _dict.get('messages')
        else:
            raise ValueError('Required property \'messages\' not present in WafGroupsResponse JSON')
        if 'result' in _dict:
            args['result'] = [WafRuleProperties.from_dict(x) for x in _dict.get('result')]
        else:
            raise ValueError('Required property \'result\' not present in WafGroupsResponse JSON')
        if 'result_info' in _dict:
            args['result_info'] = WafGroupsResponseResultInfo.from_dict(_dict.get('result_info'))
        else:
            raise ValueError('Required property \'result_info\' not present in WafGroupsResponse JSON')
        return cls(**args)

    @classmethod
    def _from_dict(cls, _dict):
        """Initialize a WafGroupsResponse object from a json dictionary."""
        return cls.from_dict(_dict)

    def to_dict(self) -> Dict:
        """Return a json dictionary representing this model."""
        _dict = {}
        if hasattr(self, 'success') and self.success is not None:
            _dict['success'] = self.success
        if hasattr(self, 'errors') and self.errors is not None:
            _dict['errors'] = self.errors
        if hasattr(self, 'messages') and self.messages is not None:
            _dict['messages'] = self.messages
        if hasattr(self, 'result') and self.result is not None:
            _dict['result'] = [x.to_dict() for x in self.result]
        if hasattr(self, 'result_info') and self.result_info is not None:
            _dict['result_info'] = self.result_info.to_dict()
        return _dict

    def _to_dict(self):
        """Return a json dictionary representing this model."""
        return self.to_dict()

    def __str__(self) -> str:
        """Return a `str` version of this WafGroupsResponse object."""
        return json.dumps(self.to_dict(), indent=2)

    def __eq__(self, other: 'WafGroupsResponse') -> bool:
        """Return `true` when self and other are equal, false otherwise."""
        if not isinstance(other, self.__class__):
            return False
        return self.__dict__ == other.__dict__

    def __ne__(self, other: 'WafGroupsResponse') -> bool:
        """Return `true` when self and other are not equal, false otherwise."""
        return not self == other

class WafRuleProperties():
    """
    waf rule properties.

    :attr str id: (optional) ID.
    :attr str name: (optional) Name.
    :attr str description: (optional) Description.
    :attr int rules_count: (optional) Number of rules.
    :attr int modified_rules_count: (optional) Number of modified rules.
    :attr str package_id: (optional) Package ID.
    :attr str mode: (optional) Mode.
    :attr List[str] allowed_modes: (optional) Allowed Modes.
    """

    def __init__(self,
                 *,
                 id: str = None,
                 name: str = None,
                 description: str = None,
                 rules_count: int = None,
                 modified_rules_count: int = None,
                 package_id: str = None,
                 mode: str = None,
                 allowed_modes: List[str] = None) -> None:
        """
        Initialize a WafRuleProperties object.

        :param str id: (optional) ID.
        :param str name: (optional) Name.
        :param str description: (optional) Description.
        :param int rules_count: (optional) Number of rules.
        :param int modified_rules_count: (optional) Number of modified rules.
        :param str package_id: (optional) Package ID.
        :param str mode: (optional) Mode.
        :param List[str] allowed_modes: (optional) Allowed Modes.
        """
        self.id = id
        self.name = name
        self.description = description
        self.rules_count = rules_count
        self.modified_rules_count = modified_rules_count
        self.package_id = package_id
        self.mode = mode
        self.allowed_modes = allowed_modes

    @classmethod
    def from_dict(cls, _dict: Dict) -> 'WafRuleProperties':
        """Initialize a WafRuleProperties object from a json dictionary."""
        args = {}
        if 'id' in _dict:
            args['id'] = _dict.get('id')
        if 'name' in _dict:
            args['name'] = _dict.get('name')
        if 'description' in _dict:
            args['description'] = _dict.get('description')
        if 'rules_count' in _dict:
            args['rules_count'] = _dict.get('rules_count')
        if 'modified_rules_count' in _dict:
            args['modified_rules_count'] = _dict.get('modified_rules_count')
        if 'package_id' in _dict:
            args['package_id'] = _dict.get('package_id')
        if 'mode' in _dict:
            args['mode'] = _dict.get('mode')
        if 'allowed_modes' in _dict:
            args['allowed_modes'] = _dict.get('allowed_modes')
        return cls(**args)

    @classmethod
    def _from_dict(cls, _dict):
        """Initialize a WafRuleProperties object from a json dictionary."""
        return cls.from_dict(_dict)

    def to_dict(self) -> Dict:
        """Return a json dictionary representing this model."""
        _dict = {}
        if hasattr(self, 'id') and self.id is not None:
            _dict['id'] = self.id
        if hasattr(self, 'name') and self.name is not None:
            _dict['name'] = self.name
        if hasattr(self, 'description') and self.description is not None:
            _dict['description'] = self.description
        if hasattr(self, 'rules_count') and self.rules_count is not None:
            _dict['rules_count'] = self.rules_count
        if hasattr(self, 'modified_rules_count') and self.modified_rules_count is not None:
            _dict['modified_rules_count'] = self.modified_rules_count
        if hasattr(self, 'package_id') and self.package_id is not None:
            _dict['package_id'] = self.package_id
        if hasattr(self, 'mode') and self.mode is not None:
            _dict['mode'] = self.mode
        if hasattr(self, 'allowed_modes') and self.allowed_modes is not None:
            _dict['allowed_modes'] = self.allowed_modes
        return _dict

    def _to_dict(self):
        """Return a json dictionary representing this model."""
        return self.to_dict()

    def __str__(self) -> str:
        """Return a `str` version of this WafRuleProperties object."""
        return json.dumps(self.to_dict(), indent=2)

    def __eq__(self, other: 'WafRuleProperties') -> bool:
        """Return `true` when self and other are equal, false otherwise."""
        if not isinstance(other, self.__class__):
            return False
        return self.__dict__ == other.__dict__

    def __ne__(self, other: 'WafRuleProperties') -> bool:
        """Return `true` when self and other are not equal, false otherwise."""
        return not self == other
